/*
*  Rubrica
*  file: addressbook.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "rubrica.h"
#include "main.h"
#include "addressbook.h"
#include "interface.h"
#include "dialoghi.h"
#include "file.h"
#include "file_util.h"


#define RUBRICA_TEST(number)  if ((number) < 0) \
                                 g_warning("richiesta una rubrica non esistente");


GList *book  = NULL;          /* list of addressbooks */
GtkWidget *notebook;
gint pages   = 0;             /* pages count the number of all notebook pages,
				 each page is an addressbook.  */



/*
 *
 *  Make a new notebook page that contains a cards clist and append it at 
 *  notebook end. Allocate memory for new addressbook item and fill 
 *  infos field with the "notebooks" value.
 * 
 *  return the number of the new page
 *
 */
gint
rubrica_addressbook_new(gchar *filename)
{
  AddressBook *ab;          /* the new addressbook item */
  GtkWidget *notebook,      /* */     
    *child,                 /* notebook child, this will contain cards clist */
    *label,                 /* notebook tab label */
    *clist;                 /* the cards clist */
  gint page;                /* the # of the new notebook's page */
  gchar *path, *fname = NULL;

  if (filename)
    {
      path  = g_dirname(filename);
      fname = g_basename(filename);
    }
  else
    path = NULL;

  /* do new addressbook, init and insert it in addressbooks 
     list. It will be related to new notebook page  */
  ab = (AddressBook *) g_malloc(sizeof(AddressBook));

  notebook = rubrica_addressbook_get_notebook();

  /* save current page number */
  page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
  rubrica_set_current_page(page);

  /* make notebook page's child */
  child = rubrica_interface_insert_new_view(&clist);

  /* tab label 
     NOTA:
     l'etichetta da inserire nel notebook, NON DEVE ESSERE MOSTRATA
     DON'T SHOW the label that will be inserted in notebook  
  */  
  if (!fname)
    fname = _("no name");
  label = gtk_label_new (fname);  

  /* append new notebook page */
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child, label);    

  /* get the number of this new page */
  page = gtk_notebook_page_num(GTK_NOTEBOOK(notebook), child);  

  /* store infos about this addressbook/page */
  ab->rubrica      = NULL;
  ab->modified     = FALSE;
  ab->empty        = TRUE; 
  ab->new          = TRUE;
  ab->name         = g_strdup(fname);
  ab->path         = g_strdup(path);
  ab->notebook     = notebook;
  ab->child        = child;
  ab->clist        = clist;
  ab->label        = label;
  ab->item_in_list = 0;
  ab->id           = page;
  rubrica_addressbook_append(ab);

  /* switch to page number "page" */
  gtk_notebook_set_page(GTK_NOTEBOOK(notebook), page); 

  /* init views */
  rubrica_addressbook_init_view();

  /* increase pages counter */
  rubrica_addressbook_inc_pages();
 
  /* return the number of new page */
  return (gint) page;
}

/* 
 *   init addressbook 
 */
void rubrica_addressbook_init(void)
{
  /*     the first addressbook of addressbooks.    
   */
  AddressBook *ab;

  ab = g_malloc(sizeof(AddressBook));

  ab->rubrica = NULL;
  ab->modified = FALSE;
  ab->empty = TRUE;
  ab->new = TRUE;

  ab->name = NULL;
  ab->path = NULL;

  ab->notebook = NULL;
  ab->child = NULL;
  ab->clist = NULL;
  ab->label = NULL;

  ab->item_in_list = 0;
  ab->id = 0;
  
  book  = g_list_append(book, ab);  
}

/* 
 * init an addressbook item 
 */
void rubrica_addressbook_init_item(AddressBook *ab)
{
  ab->rubrica  = NULL;
  ab->modified = FALSE;
  ab->empty    = TRUE;
  ab->new      = TRUE;

  ab->name = NULL;
  ab->path = NULL;

  ab->item_in_list = 0;
  ab->id           = 0;
}


/*
 *  clean the addressbook view
 */ 
void rubrica_addressbook_init_view(void)
{
  GtkCList *dati;
  
  /* pulisco la clist dati  */
  dati = rubrica_vista_get_dati(); 
  gtk_clist_clear(GTK_CLIST(dati));    
}


/* 
 *  insert a new addressbook in book 
 */
void rubrica_addressbook_append(AddressBook *ab)
{
  book  = g_list_append(book, ab);
}

/* 
 *  ritorna la rubrica associata alla pagina "page" del notebook
 *  return the addressbook in position number page 
 */
AddressBook *rubrica_addressbook_get_addressbook(gint page)
{
  GtkWidget *notebook, *child;
  AddressBook *ab;
  GList *alias = book;

  RUBRICA_TEST(page);

  if (rubrica_addressbook_get_pages() == 0)
    return (AddressBook *) alias->data;

  notebook = rubrica_addressbook_get_notebook();
  child = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), page);

  while (alias)
    {
      ab = (AddressBook *) alias->data;

      if (ab->child == child)
	return (AddressBook *) ab;
      else
	alias = g_list_next(alias);     
    }
  return NULL;
}


/*
 *
 *  copia il contenuto delle variabili rubrica, clist, ecc, relative al 
 *  addressbook selezionato nelle corrispondenti variabili globali
 *
 *  copy the content of addressbook number page to global variables.
 *
*/
void rubrica_addressbook_apply(gint page)
{
  AddressBook *ab;
  gint item;

  /* book is a list of lists and contains all addressbooks opened. 
     Each notebook page is one addressbook. Data contained in list item
     contains ptr to real address, plus some infos about the addressbook
  */
  
  /* get data from position "page" */
  ab =  rubrica_addressbook_get_addressbook(page);
  
  if (!ab)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");

      return;
    }

  /* get the number of items in addressbook */
  item = g_list_length(ab->rubrica); 
  if (item != ab->item_in_list)
    ab->item_in_list = item;

  /* set global variables to ab content */
  rubrica_set_rubrica(ab->rubrica);
  
  if (ab->rubrica)
    rubrica_set_new(FALSE);
  else
    rubrica_set_new(TRUE);
  
  rubrica_modificata(ab->modified);
  rubrica_set_empty(ab->empty);
  rubrica_file_set_names(ab->path, ab->name);

  if (!ab->clist)
    {
      GtkWidget *nb = NULL;
      GtkWidget *child;
      
      nb = rubrica_addressbook_get_notebook();
      child = gtk_notebook_get_nth_page(GTK_NOTEBOOK(nb), page);
      
      ab->clist = gtk_object_get_data(GTK_OBJECT(child),"cards"); 
    }

  rubrica_vista_set_schede(ab->clist);
  rubrica_vista_set_indirizzi(ab->item_in_list);    
  
  if (rubrica_file_is_start())
    rubrica_file_set_start(FALSE);
}

/* 
 * return the number of addressbooks 
 */
inline gint 
rubrica_addressbook_get_pages(void)
{
  return pages;
}


/* 
 * set addressbook number to n 
 */
inline void 
rubrica_addressbook_set_page(gint n)
{
  pages = n;
}


/* 
 * add an addressbook 
 */
inline void 
rubrica_addressbook_inc_pages(void)
{
  pages++;
}


/* 
 * delete an addressbook 
 */
inline void 
rubrica_addressbook_dec_pages(void)
{
  pages--;
}

/* 
 * save the notebook ptr
 */
void rubrica_addressbook_set_notebook(GtkWidget *nb) 
{
  notebook = nb;
}

/* 
 * get the notebook ptr
 */
GtkWidget *rubrica_addressbook_get_notebook(void)
{
  return (GtkWidget*) notebook;
}

/*
 * store the new ptr to rubrica if the old one was changed
 */
void rubrica_addressbook_set_rubrica(GList *rub, gint page)
{
  AddressBook *data;
  
  data = rubrica_addressbook_get_addressbook(page);  
  
  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");
      
      return;
    }

  data->rubrica = rub;
}


/*
 *  save the ptr to rubrica and rubrica's state (modified, empty) of 
 *  addressbook contained in page number "pos"
 *
 */
void rubrica_addressbook_set_info(GList *rub, gboolean mod, gboolean emp, 
				  gboolean new, gint pos)
{
  AddressBook *data;

  data = rubrica_addressbook_get_addressbook(pos);

  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");

      return;
    }
  
  if (rub != NULL)
    data->rubrica = rub;

  data->modified = mod;
  data->empty = emp;
  data->new = new;
  data->item_in_list = g_list_length(rub); 
  data->id = pos;

}

/*
 *  save widget info of addressbook contained in page number "pos" 
 */
void rubrica_addressbook_set_widget_info(GtkWidget *notebook, GtkWidget * child, 
					 GtkWidget *clist, GtkWidget *label, 
					 gint page)
{
  AddressBook *data;
  
  data = rubrica_addressbook_get_addressbook(page);  
  
  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");

      return;
    }
  
  data->notebook = notebook;
  data->child = child;
  data->clist = clist;
  data->label = label;  

  data->id = page;
}


/* 
 * return path and filename of addressbook number page
 * if the selected page don't contain addressbook then return NULL
 *
 */
void rubrica_addressbook_get_path_and_name(gchar **path, gchar **name, gint page)
{ 
  AddressBook *data;

  RUBRICA_TEST(page);

  data = rubrica_addressbook_get_addressbook(page);

  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");
      
      *path = NULL;
      *name = NULL;
      return;
    }

  *path = g_strdup(data->path);
  *name = g_strdup(data->name);
}

/* 
   save path and name in addressbook number page
*/
void rubrica_addressbook_set_path_and_name(gchar *path, gchar *name, gint page)
{
  AddressBook *data;

  RUBRICA_TEST(page);

  /* get data of the nth item of book (an AddressBook) and replace name and path */
  data = rubrica_addressbook_get_addressbook(page);

  if (!data)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");
      
      return;
    }

  if (data->name)
    g_free(data->name);
  data->name = g_strdup(name);
  
  if (data->path)
    g_free(data->path);
  data->path = g_strdup(path);

  /* show new filename in notebook tab */
  gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(data->notebook), 
				  data->child, data->name);
}


void rubrica_addressbook_close(gpointer notebook)
{  
  AddressBook *ab;
  gint page, scelta;
  gchar *mes;
  
  page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
  ab = rubrica_addressbook_get_addressbook(page);
  
  if (!ab)
    {
      g_warning("errore, nessun addressbook alla pagina richiesta");
      
      return;
    }

  if (ab->modified)
    {
      mes = g_strdup(_("Warning. Address book was changed.\nChanges not saved, will be lose.\nContinue?")); 
      
      scelta = confirm_file_close(mes);
      g_free(mes);
      
      switch(scelta)
	{
	case 0:
	  rubrica_addressbook_real_close(ab, notebook, page);
	  break;
	  
	case 1:
	default:
	  break;
	}  
    }

  rubrica_addressbook_real_close(ab, notebook, page);
}

void rubrica_addressbook_real_close(AddressBook *ab, gpointer notebook, gint page)
{
  /* per chiudere un file, devo liberare la memoria che occupa
     devo estrarre da book (la lista degli addressbook) l'elemento che 
     contiene il ptr alla vera rubrica e liberare la memoria occupata.
     Quindi liberare la memoria occupata dal nodo
  */
  
  /* page is the number of current page */  
  GList *nodo;
  GtkWidget *cards;
  gint pages;

  pages = rubrica_addressbook_get_pages();  /* get the number of pages in notebook */

  /* if notebook has 2 or more pages, then delete notebook's page #page */
  if (pages > 1)    
    {
      /* find the node that conytains the addressbook */
      nodo = g_list_find(book, ab);
      
      /*     Free the memory
       */
      gtk_notebook_remove_page(GTK_NOTEBOOK(notebook), 
			       page);                /* delete notebook's page    */
      rubrica_svuota_memoria(ab->rubrica);           /* delete rubrica's contents */
      if (ab->name)                                  /* delete path and filename  */
	g_free(ab->name);                             
      if (ab->path)
	g_free(ab->path);

      /*   are deleted other widgets by gtk_notebook_remove_page ?
	   
	   gtk_widget_unref(ab->label);
	   gtk_widget_unref(ab->clist);
	   gtk_widget_unref(ab->child);
      */
      
      if (nodo)  /* if nodo == NULL, do nothing */
	{
	  book = g_list_remove_link(book, nodo);    /* extract node from list */
	  g_list_free_1(nodo);                      /* delete it */
	}
      rubrica_addressbook_dec_pages();          /* and update pages counter */
    }
  else  /* notebook has only one page (page number 0) */
    {
      cards = ab->clist;
      gtk_clist_clear(GTK_CLIST(cards));           /* clear cards clist   */
      rubrica_addressbook_init_item(ab);           /* init data structure */
      gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook), 
				      ab->child, _("no name"));  
      rubrica_addressbook_apply(0);    /* copy data to globals variables */
      rubrica_file_set_start(TRUE);    /* first notebook page */
    }
}









