/*
*  Rubrica
*  file: file.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "rubrica.h"
#include "modulo.h"
#include "main.h"
#include "file.h"
#include "file_util.h"
#include "prefer.h"
#include "dictionary.h"
#include "addressbook.h"

VistaDati vista;
gchar *path;
gchar *nomefile;
Dictionary dict;
gchar *language;
gboolean language_tested = FALSE;
gboolean recover = FALSE;


void rubrica_file_init_names(void)
{
  if (path)
    g_free(path);
  if (nomefile)
    g_free(nomefile);

  path = NULL;
  nomefile = NULL;
}


void rubrica_file_set_names(gchar *pathfile, gchar *filename)
{
  if (path)
    g_free(path);
  if (nomefile)
    g_free(nomefile);

  path = g_strdup(pathfile);
  nomefile = g_strdup(filename);   
}


void 
rubrica_file_salva(void)
{
  GtkWidget *app;
  gchar *tmp;

  app = rubrica_get_app();
  
  /* get dictionary  */
  //  dict = rubrica_get_dictionary("EN");

  /* 
     se non c' un nome file di default allora bisogna richiamare salva_con_nome     
     if rubrica_salva_file is called without a file name, then call save as
  */
  if (!nomefile || g_strcasecmp(nomefile, _("no name")) == 0)
    /* salvo come default? */
    rubrica_file_salva_come(NULL, NULL);

    /*   altrimenti provo a scrivere il file 
    */
  else 
    if (!rubrica_file_write_xmlfile(nomefile))
      {
	tmp = g_strdup_printf(_("I can't save the file %s"),nomefile);
	gnome_app_error(GNOME_APP(app), tmp);
	g_free(tmp);
      }
}

void 
rubrica_file_salva_come(GtkWidget *w, gpointer data)
{
  GtkWidget *app;
  GtkFileSelection *fs;
  RubricaPreferences *p;
  
  app = rubrica_get_app();

  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Save as")));
  
  /* se ho gi un nome di file lo mostro nell'entry */
  if (nomefile)
       gtk_file_selection_set_filename(fs, nomefile);

  /* altrimenti, se settato, propongo il nome di default */
  else
    {
      p = rubrica_get_preferences();
      
      if (p->ab_name)
	gtk_file_selection_set_filename(fs, p->ab_name);
    }

  /* connetto i bottoni ok e cancella */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_file_scrivi_file), fs);
  
  /* con il bottone cancella distruggo direttamente il widget */
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));
  
  gtk_widget_show(GTK_WIDGET(fs));
}


void
rubrica_file_scrivi_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *tmp;
  gchar *fname=NULL;
  RubricaPreferences *p;
  AddressBook *ab;
  gint page;      /* notebook's page, now active */

  page = rubrica_get_current_page();
  ab = rubrica_addressbook_get_addressbook(page);

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));
  g_return_if_fail(ab != NULL);

  tmp = gtk_file_selection_get_filename(fs);
  p = rubrica_get_preferences();
  rubrica_set_rubrica(ab->rubrica);

  /* se il file non ha estensione o l'estensione non  .rub */
  if (!rubrica_file_has_extension(tmp))
    fname = g_strdup_printf("%s.rub",tmp);  
  else 
    fname = g_strdup(tmp);
  
  /* se devo effettuare il backup */
  if (p->backup)
    {
      gchar *newname;

      newname = g_strdup_printf("%s1", fname);
      rubrica_file_do_backup(fname, newname);
      g_free(newname);
    }

  /* se non  stato selezionato nessun file
     o non  possibile aprire il file segnalare l'errore */  
  if ( !fname || !rubrica_file_write_xmlfile(fname)) 
    {
      gchar *err;
      GtkWidget *dlg;
      
      /* il dialogo di errore ha il widget fs come genitore */
      if (fname)
	err = g_strdup_printf(_("I can't save the file %s"), fname); 
      else
	err = g_strdup_printf(_("I can't save the file")); 
      
      dlg = gnome_error_dialog_parented(err, GTK_WINDOW(fs));
      gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);   
      g_free(err);
    }
  else /* il file  stato salvato */
    {      
      rubrica_addressbook_set_info(NULL, FALSE, FALSE, FALSE, page);
      rubrica_addressbook_set_path_and_name(g_dirname(fname), g_basename(fname), page);
      
      /* imposto i nuovi filename */
      rubrica_file_set_names(g_dirname(fname), g_basename(fname));
      
      /* se devo mostrare la path del file, aggiorno la label con la path */
      if (preferences_get_show_path())
	rubrica_vista_set_str_label_path(path);
      else
	/* altrimenti la agiorno con il nome del file */
	rubrica_vista_set_str_label_path(nomefile);

      /* in ogni caso, mostro il nome del file */
//      rubrica_vista_set_str_label_file(nomefile);
      
      rubrica_addressbook_apply(page);

      gtk_widget_destroy(GTK_WIDGET(fs));
    }

  g_free(fname);
}

gboolean
rubrica_file_write_xmlfile(gchar *name)
{
  xmlDocPtr doc;  /* puntatore al documento xml */
  GList *rub;     /* ptr to rubrica, this contains data that we want save */
  GList *l;       /* alias to rub */
  gchar *path, *fname;

  rub = rubrica_get_rubrica();
  language = preferences_get_language();

  g_return_val_if_fail(name != NULL, FALSE);
  g_return_val_if_fail(rub != NULL, FALSE);
  
  /* se l'utente vuole salvare le rubriche in una cartella predefinita */
  /*
   *           SIMULO LA SCELTA DI SALVARE LA RUBRICA IN UNA CARTELLA DI DEFAULT
   */
  if (TRUE)  
    {
      name = g_basename(name);
      path = preferences_get_path();
      fname = g_strdup_printf("%s/%s", path, name);
    }
  else
    fname = g_strdup(name);

  /* creazione di un nuovo documento xml */
  doc = xmlNewDoc("1.0");
  
  /* creo un nuovo nodo radice rubrica */
  doc->root = xmlNewDocNode(doc, NULL, "Rubrica", NULL);

  /*      Set Language and Get dictionary
   */
  if (strcmp(language, "Italian") == 0)
    {
      xmlNodeSetLang(doc->root,"IT");
      dict = rubrica_get_dictionary("IT");    
    }
  else
    if (strcmp(language, "English") == 0)
      {
	xmlNodeSetLang(doc->root,"EN");
	dict = rubrica_get_dictionary("EN");    
      }
    else
      {
	xmlNodeSetLang(doc->root,"OTHER");
	dict = rubrica_get_dictionary(NULL);    
      }

  /* attraverso la rubrica e scrivo i dati */
  for (l = rub; l != NULL; l = l->next) {
    RubricaItem *item = l->data;
    
    rubrica_file_build_xmlnode(doc->root, item);
  }
    
  if (g_file_exists(fname))
    {
      if (preferences_get_do_backup())
	rubrica_file_do_backup(fname, fname);
      else
	remove(fname);
    }
  
  if (xmlSaveFile(fname, doc) == -1) 
    {
      xmlFreeDoc(doc);
      return FALSE;
    }  
  /* if we are here the addressbook is write */
  g_print("\nfile has been saved");
//  xmlFreeDoc(doc);    
  g_free(fname);

  return TRUE;
}

void 
rubrica_file_build_xmlnode(xmlNodePtr xmlnode, RubricaItem *item)
{
  GList *alias;
  xmlNodePtr newxml;

  xmlNodePtr schedaxml;
  xmlNodePtr datixml;
  xmlNodePtr webxml;
  xmlNodePtr mailxml;
  xmlNodePtr telxml;
  xmlNodePtr notexml;

  xmlNodePtr nodo;

  g_return_if_fail(xmlnode != NULL);
  g_return_if_fail(item != NULL);
  
  /*
    newxml  figlio di xmlnode (il primo xmlnode  doc->root)
    boolxml, schedaxml, datixml, ..., sono figli di newxml
  */
    
  newxml = xmlNewChild(xmlnode, NULL, dict.address, NULL);
//  xmlSetProp(xmlnode, "rescue", "ADDRESS");
  
  /*
    xmlNewTextChild 
    sintassi :
         - nodo genitore del nodo creato con la chiamata
	 - namespace
	 - nome del nodo creato
	 - testo (elemento) da inserire nel nodo creato
  */
  schedaxml = xmlNewTextChild(newxml, NULL, dict.card, item->scheda);
//  xmlSetProp(schedaxml, "rescue", "CARD");

  /* setto le propriet della scheda (editabile, concellabile) */
  if (item->editabile)
    xmlSetProp(schedaxml, dict.mod, "TRUE");
  else
    xmlSetProp(schedaxml, dict.mod, "FALSE");  
  if (item->cancellabile)
    xmlSetProp(schedaxml, dict.del, "TRUE");
  else
    xmlSetProp(schedaxml, dict.del, "FALSE");
  
  /* ho creato l'elemento xml es.: 
     <Scheda editabile="TRUE" cancellabile="FALSE>scheda numero uno</Scheda>
     <Card modifying="TRUE" deleting="FALSE">card number one</Card>
  */

  datixml = xmlNewTextChild(newxml, NULL, dict.data, NULL);
//  xmlSetProp(nodo, "rescue", "PERSONAL_DATA");
  nodo = xmlNewTextChild(datixml, NULL, dict.name, item->nome);
//  xmlSetProp(nodo, "rescue", "FIRST_NAME");
  nodo = xmlNewTextChild(datixml, NULL, dict.surn, item->cognome);
//  xmlSetProp(nodo, "rescue", "LAST_NAME");
  nodo = xmlNewTextChild(datixml, NULL, dict.prof, item->professione);
//  xmlSetProp(nodo, "rescue", "PROFESSION");
  nodo = xmlNewTextChild(datixml, NULL, dict.str, item->via);
//  xmlSetProp(nodo, "rescue", "STREET");
  nodo = xmlNewTextChild(datixml, NULL, dict.num, item->numero);
//  xmlSetProp(nodo, "rescue", "STREET_NUMBER");
  nodo = xmlNewTextChild(datixml, NULL, dict.zip, item->cap);
//  xmlSetProp(nodo, "rescue", "ZIP_CODE");
  nodo = xmlNewTextChild(datixml, NULL, dict.city, item->citta);
//  xmlSetProp(nodo, "rescue", "CITY");
  nodo = xmlNewTextChild(datixml, NULL, dict.prov, item->provincia);
//  xmlSetProp(nodo, "rescue", "PROVINCE");
  nodo = xmlNewTextChild(datixml, NULL, dict.st, item->stato);
//  xmlSetProp(nodo, "rescue", "COUNTRY");
  
  webxml = xmlNewChild(newxml, NULL, dict.webadd, NULL);
//  xmlSetProp(webxml, "rescue", "WEB_ADDRESS");

  alias = item->web_list;
  while(alias)
    {
      nodo = xmlNewTextChild(webxml, NULL, dict.web, alias->data);
//      xmlSetProp(nodo, "rescue", "WEB");

      alias = alias->next;
    }
  
  mailxml = xmlNewChild(newxml, NULL, dict.mailadd, NULL);
//  xmlSetProp(mailxml, "rescue", "EMAIL_ADDRESS");

  alias = item->mail_list;
  while(alias)
    {
      nodo = xmlNewTextChild(mailxml, NULL, dict.mail, alias->data);
//      xmlSetProp(nodo, "rescue", "EMAIL");
  
      alias = alias->next;
    }

  telxml = xmlNewChild(newxml, NULL, dict.tellist, NULL);
//  xmlSetProp(telxml, "rescue", "TELEPHONE_NUMBERS");
  alias = item->tel_list;
  while(alias)
    {
      TelNum *tt = (TelNum*) alias->data;

      nodo = xmlNewTextChild(telxml, NULL, dict.telnum, tt->numero_telefono);      
//      xmlSetProp(nodo, "rescue", "TELEPHONE");
      xmlSetProp(nodo, dict.teltype, tt->tipo);
      
      alias = alias->next;
    }
  
  notexml = xmlNewTextChild(newxml, NULL, dict.note, item->note);
//  xmlSetProp(notexml, "rescue", "NOTE");
}






/*      Load Routines
*/
/*  this function is called when user want start rubrica whit one file
*/
void rubrica_file_start_with_file(gchar *path, gchar *file)
{
  gchar *filename;
  gboolean loaded;
  
  filename = g_strdup_printf("%s/%s", path, file);
  rubrica_file_set_start(TRUE);

  /* load file */
  loaded = rubrica_file_load_xmlfile(filename);
  if (!loaded)
    {
      g_warning("I can't load default addressbook. ");
    }
  
  g_free(filename);
}

gboolean rubrica_file_load_xmlfile(gchar *filename)
{
  gboolean use_dict;  
  GList *rub;
  gint page;
  gchar *fpath, *fname;

  /* if user want to use his dictionary */
  use_dict = preferences_get_use_dictionary();

  if (use_dict)
    {
      /* if dictionary isn't loaded, then load it */
      if(!dictionary_is_loaded())
	rubrica_load_translation();  
      
      /* get dictionary (from memory, only choose one dictionary)

	 NULL - load other (user) dictionary
	 IT   - load italian dictionary
	 EN   - load english dictionary
       */
      dict = rubrica_get_dictionary(NULL);
    }

  /* real file load */
  if (!rubrica_file_real_load_xmlfile(filename)) 
    return FALSE;
  else
    {
      /* file has been loaded 
	 look preferences, if I have to do somethinks
       */

      if (preferences_get_show_path())
	  /*   user want to display path
	       path and nomefile are globals in file.c
	  */
	  rubrica_vista_set_str_label_path(path);      	
      else
	  rubrica_vista_set_str_label_path(NULL);

      rubrica_vista_set_str_label_file(nomefile);
      rubrica_set_empty(FALSE);      
    }
  /* if we are here, then file has been loaded and preferences 
     have been setted, so we need to store some information about 
     this file, and addressbook strored in memory
  */

  /* get ptr to rubrica (addressbook just read) */
  rub = rubrica_get_rubrica();
  page = rubrica_get_current_page();

  fpath = g_dirname(filename);
  fname = g_basename(filename);
  rubrica_addressbook_set_info(rub, FALSE, FALSE,FALSE, page);
  rubrica_addressbook_set_path_and_name(fpath, fname, page); 

  g_free(fpath);
  return TRUE;
}


void 
rubrica_file_carica_file(void)
{
  GtkFileSelection *fs;
  GtkWidget *app;

  app = rubrica_get_app();

  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Open file...")));
  
  /* connetto i bottoni ok e cancel */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_file_apri_file), fs);
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  /* visualizzo la finestra in posizione centrale */
  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);

  /* il genitore della finestra fileselection  app */
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));

  gtk_widget_show(GTK_WIDGET(fs));
}

void 
rubrica_file_apri_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *fname;
  gchar *tmp;
  gboolean loaded;

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));

  fname = gtk_file_selection_get_filename(fs);
  loaded = rubrica_file_load_xmlfile(fname);
  if (!loaded)
    {
      GtkWidget *dlg;
      
      tmp = g_strdup_printf(_("I can't read the file %s"), fname);
      
      dlg = gnome_error_dialog_parented(tmp,GTK_WINDOW(fs));
      gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);

      g_free(tmp);
    }
  else
    gtk_widget_destroy(GTK_WIDGET(fs));
}


/* 
 *  read the addressbook file 
 */
gboolean 
rubrica_file_real_load_xmlfile(gchar *fname)
{
  xmlDocPtr doc;
  gchar *lang; /* language set in file */

  g_return_val_if_fail(fname != NULL, FALSE);
  
  /* file not exist */
  if (!g_file_exists(fname))
    return FALSE;

  /* read and parse file */
  doc = xmlParseFile(fname);

  if (!doc)
    return FALSE;

  if (!doc->root || !doc->root->name || 
      g_strcasecmp(doc->root->name, "Rubrica") != 0)
    {
      xmlFreeDoc(doc);
      return FALSE;
    }
  lang = xmlNodeGetLang(doc->root);

  if(!lang)
    {
      /* may be this file has been created with an rubrica version less then 0.5
	 Those version use english as language
      */
      recover = TRUE;
      g_warning("read_xml_file - recover mode:: salvare il file");     
    }
  else
    {
      if (strcmp(lang, "IT") == 0)
	dict = rubrica_get_dictionary("IT");
      else     
	if (strcmp(lang, "EN") == 0)
	  dict = rubrica_get_dictionary("EN");
	else
	  dict = rubrica_get_dictionary(NULL);
    }

  /* parse doc, extract address from file and put them in memory */
  rubrica_file_parse_doc(doc, fname);

  /* addressbook has been parsed. Free memory */
  xmlFreeDoc(doc);
 
  rubrica_file_set_names(g_dirname(fname),g_basename(fname)); 
 
  return TRUE;  
}


void 
rubrica_file_parse_doc(xmlDocPtr doc, gchar *fname)
{  
//  GtkWidget *app;
  xmlNodePtr nodo;
  gint page = 0;

  /* inizializzo le strutture necessarie per 
     caricare una nuova lista
  */
//  app = rubrica_get_app();

  if (rubrica_file_is_start())
    rubrica_addressbook_apply(0);
  else
    page = rubrica_addressbook_new(fname);
    
  rubrica_set_current_page(page);
    

  /* try to load an old or damaged file
   */
  if(recover)
    {  
      xmlNodePtr xmltmp;

      xmltmp = doc->root->childs;
      if (g_strcasecmp(xmltmp->name, "Address") == 0)
	dict = rubrica_get_dictionary("EN");
      if (g_strcasecmp(xmltmp->name, "Indirizzo") == 0)
	dict = rubrica_get_dictionary("IT");
      else
	{	/* procedura disperata, sottoponi all'utente il file */ 

	}
    }

  /* prendo il nodo Indirizzo dal file e lo inserisco nella (nuova) rubrica 
     il primo nodo  doc->root->childs == xmlnew (salvato)
  */
  for(nodo = doc->root->childs; nodo != NULL; nodo=nodo->next)    
    rubrica_file_load_from_xmlnode(nodo, NULL);  
}

void
rubrica_file_load_from_xmlnode(xmlNodePtr xmlnode, GList *l)
{
  RubricaItem *r;  
  gboolean load_rescue = FALSE;
  gchar *buffer;
  GList *web, *mail, *tel;
  xmlNodePtr figlio;
  /* i nodi xmlnode sono nodi xmlnew
     - devo estrarre i figli di xmlnew == xmldoc che sono:
       schedaxml, ...
  */

  if (!xmlnode->name ||                               /* se il nodo non ha nome o */
      g_strcasecmp(xmlnode->name, dict.address) != 0) /* il nome non  Indirizzo  */
    {
      /* We found an error. We have loaded onje dictionary, but 
	 xmlnode->name and dict.address don't match. 
	 Will do:
	 - Try other dictionary 
	 - if fail try to recover file with user help
      */
      if (xmlnode->name)
	{
	  gboolean dict_found = FALSE;

	  /* try italian dictionary */
	  dict = rubrica_get_dictionary("IT");
	  if (g_strcasecmp(xmlnode->name, dict.address) == 0)
	    dict_found = TRUE;	  /* OK found */
	  else
	    {
	      /* try english dictionary */
	      dict = rubrica_get_dictionary("EN");	      
	      if (g_strcasecmp(xmlnode->name, dict.address) == 0)
		dict_found = TRUE;  /* OK found */
	      else
		{
		  /* try user dictionary */
		  dict = rubrica_get_dictionary(NULL);	      
		  if (g_strcasecmp(xmlnode->name, dict.address) == 0)
		    dict_found = TRUE;	  /* OK found */
		}
	    }
	  if (!dict_found)	    
	    load_rescue = TRUE;	    
	}      
    }
    
  r = g_malloc(sizeof(RubricaItem));



  /* schedaxml */
  figlio = xmlnode->childs;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.card) != 0)    
    r->scheda = "";    
  else     
    r->scheda = g_strdup(xmlNodeGetContent(figlio));
  
  /* scheda ha degli attributi, bisogna estrarli */
  buffer = xmlGetProp(figlio, dict.mod);
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->editabile = TRUE;
  else 
    r->editabile = FALSE;
  
  buffer = xmlGetProp(figlio, dict.del);
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->cancellabile = TRUE;
  else 
    r->cancellabile = FALSE;  
  
  /* dati personali -- datixml -- */
  figlio = figlio->next; 
  if (!figlio->name || g_strcasecmp(figlio->name, dict.data) != 0)
    r->nome = r->cognome 
      = r->professione = r->via 
      = r->numero = r->cap 
      = r->citta = r->provincia 
      = r->stato = "";
  else 
    {
      /* carico i figli di datixml
	 - datixml=figlio 
	 - nodo = figlio di datixml
      */
      xmlNodePtr nodo;
      gchar *tmp;
      
      nodo = figlio->childs;
      tmp = xmlNodeGetContent(nodo);
      if (tmp)
	r->nome = g_strdup(tmp);
      else 
	r->nome = "";
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if (tmp)
	r->cognome = g_strdup(tmp);
      else	
	r->cognome = "";
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if (tmp)
	r->professione = g_strdup(tmp);
      else
	r->professione = "";
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if 
	(tmp) r->via = g_strdup(tmp);
      else      
	r->via = ""; 
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if 
	(tmp) r->numero  = g_strdup(tmp);
      else      
	r->numero = "";
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if 
	(tmp)r->cap = g_strdup(tmp);
      else      
	r->cap = "";

      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if (tmp)
	r->citta = g_strdup(tmp);
      else      
	r->citta = "";
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if (tmp) 
	r->provincia = g_strdup(tmp);
      else     
	r->provincia = "";
      
      nodo = nodo->next;
      tmp = xmlNodeGetContent(nodo);
      if (tmp)
	r->stato = g_strdup(tmp);
      else      
	r->stato = "";
    }
  
  /* web */
  web = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.webadd) != 0)
    {
      r->web_list = NULL;
      r->nweb = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp;
	  
	  tmp = xmlNodeGetContent(nodo);
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      web = g_list_append(web, buffer);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->web_list = web;
      r->nweb = n;
    }
  
  /* email */
  figlio = figlio->next;
  mail = NULL;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.mailadd) != 0)
    {
      r->mail_list = NULL;
      r->nmail = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp;
	  
	  tmp = xmlNodeGetContent(nodo);
	  
	  if (tmp)
	    {
	      buffer = g_strdup(tmp);
	      
	      mail = g_list_append(mail, buffer);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->mail_list = mail;
      r->nmail = n;
    }  
  
  /* telefono */
  tel = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, dict.tellist) != 0)
    {
      r->tel_list = NULL;
      r->ntel = 0;
    }
  else
    {
      gint n=0;
      xmlNodePtr nodo;
      TelNum *t;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  gchar *tmp1=NULL, *tmp2=NULL;
	  
	  tmp1 = xmlNodeGetContent(nodo);
	  tmp2 = xmlGetProp(nodo, dict.teltype);
	  
	  if (tmp1)
	    {	
	      t = g_malloc(sizeof(TelNum));
	      
	      t->numero_telefono = g_strdup(tmp1);
	      t->tipo = g_strdup(tmp2);	  	     
	      
	      tel = g_list_append(tel, t);
	      nodo = nodo->next;
	      n++;
	    }
	}
      r->tel_list = tel;
      r->ntel = n;
    }
  
  /* note */
  figlio = figlio->next;
  r->note = NULL;
  if (figlio->name && g_strcasecmp(figlio->name, dict.note) == 0)
    {
      gchar *tmp;
      
      tmp = xmlNodeGetContent(figlio);
      
      if (tmp)
	r->note = g_strdup(tmp);
    } 
    
  rubrica_inserisci_elemento(r);
  rubrica_vista_inc_indirizzi();
  rubrica_inserisci_scheda(r);
}











