/*
*  Rubrica
*  file: file_old.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "rubrica.h"
#include "main.h"
#include "addressbook.h"
#include "file_old.h"

/* 
 *                                                                   
 *     routine di compatibilit. Leggono le rubriche create            
 *     con versioni precedenti alla 0.5                                
 *                                                                     
 *     Routine for backward compatibility. Reads old rubrica files     
 *                                                                    
 */

void rubrica_old_file_apri_file(gchar *fname)
{
  gchar *tmp=NULL;
  gint page;

  g_return_if_fail(fname != NULL);

  rubrica_set_empty(TRUE);
  if (!rubrica_old_file_read_xml_file(fname)) 
    {
      /* File not opened, error reading file
      */
      GtkWidget *app;
            
      app = rubrica_get_app();
      if (fname)
	tmp = g_strdup_printf(_("I can't read %s"), fname);

      /* if (!fname) function return because fail g_return_... */
      gnome_app_error(GNOME_APP(app), tmp);
      g_free(tmp);
    }
  else /* Ok file read */
      rubrica_set_empty(FALSE);

  page = rubrica_get_current_page();
  rubrica_addressbook_set_path_and_name(g_dirname(fname),g_basename(fname), page); 
  rubrica_addressbook_set_info(rubrica_get_rubrica(), FALSE, FALSE, FALSE, page);
}

gboolean 
rubrica_old_file_read_xml_file(gchar *fname)
{
  xmlDocPtr doc;

  g_return_val_if_fail(fname != NULL, FALSE);
  
  /* se il file non esiste */
  if (!g_file_exists(fname))
    return FALSE;

  doc = xmlParseFile(fname);

  if (!doc)
    return FALSE;

  if (!doc->root || !doc->root->name || 
      g_strcasecmp(doc->root->name, "Rubrica") != 0)
    {
      xmlFreeDoc(doc);
      return FALSE;
    }
  rubrica_old_file_parse_doc(doc);

  xmlFreeDoc(doc);
 
  return TRUE;  
}

void 
rubrica_old_file_parse_doc(xmlDocPtr doc)
{  
  xmlNodePtr nodo;
  gint page;

  /* inizializzo le strutture necessarie per caricare una nuova lista
  */
  page = rubrica_addressbook_new(NULL);

  rubrica_set_current_page(page);
  /* prendo il nodo Indirizzo dal file e lo inserisco nella (nuova) rubrica 
     il primo nodo  doc->root->childs == xmlnew (salvato)
  */
  for(nodo = doc->root->childs; nodo != NULL; nodo=nodo->next)
    rubrica_old_file_load_element_from_xml(nodo, NULL);  
}

void
rubrica_old_file_load_element_from_xml(xmlNodePtr xmlnode, GList *l)
{
  gchar *buffer;
  RubricaItem *r;  
  GList *web, *mail, *tel;
  xmlNodePtr figlio;

  /* i nodi xmlnode sono nodi xmlnew
     - devo estrarre i figli di xmlnew == xmldoc che sono:
       schedaxml, ...
  */
  
  if (!xmlnode->name ||  /* se il nodo non ha nome o */
                         /* il nome non  Indirizzo  */
      g_strcasecmp(xmlnode->name, "Indirizzo") != 0) 
    return;
    
  r = g_malloc(sizeof(RubricaItem));

  /* schedaxml */
  figlio = xmlnode->childs;
  if (!figlio->name || g_strcasecmp(figlio->name, "Scheda") != 0)   
    {
      r->scheda = g_strdup("");
    }
  else     
    r->scheda = g_strdup(xmlNodeGetContent(figlio));
    
  /* scheda ha degli attributi, bisogna estrarli */
  buffer = xmlGetProp(figlio, "editabile");
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->editabile = TRUE;
  else 
    r->editabile = FALSE;
  
  buffer = xmlGetProp(figlio, "cancellabile");
  if (g_strcasecmp(buffer, "TRUE") == 0)
    r->cancellabile = TRUE;
  else 
    r->cancellabile = FALSE;  
  
  /* dati personali -- datixml -- */
  figlio = figlio->next; 
  if (!figlio->name || g_strcasecmp(figlio->name, "Dati") != 0)
    r->nome = r->cognome 
      = r->professione = r->via 
      = r->numero = r->cap 
      = r->citta = r->provincia 
      = r->stato = g_strdup("");
  else 
    {
      /* carico i figli di datixml
	   - datixml=figlio 
	   - nodo = figlio di datixml
      */
      xmlNodePtr nodo;

      nodo = figlio->childs;
      r->nome = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->cognome = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->professione = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->via = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->numero = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->cap = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->citta = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->provincia = g_strdup(xmlNodeGetContent(nodo));

      nodo = nodo->next;
      r->stato = g_strdup(xmlNodeGetContent(nodo));
    }

  /* web */
  web = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Indirizzi_Web") != 0)
    {
      r->web_list = NULL;
      r->nweb = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  buffer = g_strdup(xmlNodeGetContent(nodo));
	  
	  web = g_list_append(web, buffer);
	  nodo = nodo->next;
	  n++;
	}
      r->web_list = web;
      r->nweb = n;
    }
  
  /* mail */
  figlio = figlio->next;
  mail = NULL;
  if (!figlio->name || g_strcasecmp(figlio->name, "Indirizzi_Email") != 0)
    {
      r->mail_list = NULL;
      r->nmail = 0;
    }
  else
    {
      gint n=0;
      gchar *buffer;
      xmlNodePtr nodo;

      nodo = figlio->childs;
      while(nodo)
	{
	  buffer = g_strdup(xmlNodeGetContent(nodo));

	  mail = g_list_append(mail, buffer);
	  nodo = nodo->next;
	  n++;
	}
      r->mail_list = mail;
      r->nmail = n;
    }  
    
  /* telefono */
  tel = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Numeri_Telefonici") != 0)
    {
      r->tel_list = NULL;
      r->ntel = 0;
    }
  else
    {
      gint n=0;
      xmlNodePtr nodo;
      TelNum *t;
      
      nodo = figlio->childs;
      while(nodo)
	{
	  t = g_malloc(sizeof(TelNum));
	  
	  t->numero_telefono = g_strdup(xmlNodeGetContent(nodo));
	  t->tipo = g_strdup(xmlGetProp(nodo,"Tipo"));
	  
	  tel = g_list_append(tel, t);
	  nodo = nodo->next;
	  n++;
	}
      r->tel_list = tel;
      r->ntel = n;
    }
  
  /* note */
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Note") != 0)
    r->note = g_strdup("");
  else 
    r->note = g_strdup(xmlNodeGetContent(figlio));
  
  rubrica_inserisci_elemento(r);
  rubrica_vista_inc_indirizzi();
  rubrica_inserisci_scheda(r);
}
