/*
*  Rubrica
*  file: import.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "rubrica.h"
#include "file.h"
#include "file_old.h"
#include "main.h"
#include "error.h"
#include "import.h"
#include "import_int.h"
#include "import_gcard.h"
#include "addressbook.h"

ImportFrom importfrom;

static gchar *gcard_default="/.gnome/GnomeCard.gcrd";
static gchar *card_kde1="/.kde/share/apps/kab/addressbook.database";
static gchar *card_kde2="/.kde/share/apps/kab/";


void rubrica_import_init_item(RubricaItem *r)
{
  r->editabile = r->cancellabile = TRUE;      

  r->scheda = r->nome = r->cognome = r->professione = r->via = NULL;
  r->numero = r->cap = r->citta = r->provincia = r->stato = r->note = NULL;

  r->web_list = NULL;
  r->mail_list = NULL;
  r->tel_list = NULL;      

  r->nweb = r->nmail = r->ntel = 0;
}

void rubrica_import_from_init(void)
{
  importfrom = GCARD; /* import default will be csv file */
}

void rubrica_import_from_set(ImportFrom i)
{
  importfrom = (ImportFrom) i;
}

ImportFrom rubrica_import_from_get(void)
{
  return (ImportFrom) importfrom;
}


void rubrica_import_data(void)
{
  GtkWidget *fentry, *app;
  gchar *filename;
  gchar *buffer, *titolo = NULL;
  gint n, page;
  
  n=0;
  app = rubrica_get_app();
  fentry = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY
				      (rubrica_import_get_file_entry()));

  filename = gtk_entry_get_text(GTK_ENTRY(fentry));

  page = rubrica_addressbook_new(NULL);
  
  rubrica_set_current_page(page);
  switch (importfrom)
    {
    case GCARD:
      /* 	 if no name is set in gnome_entry, then 
		 load default gnomecard addressbook 
      */
      if (strcmp(filename,"") == 0)
	{
	  /* carico il file di default */
	  buffer = g_get_home_dir();
	  
	  filename = g_strdup_printf("%s%s",buffer,gcard_default);
	  rubrica_import_open_file(filename);
	  titolo = _("Imported from GnomeCard");
	  n = rubrica_vista_get_indirizzi();
	}          
      else	
	/* import gnomecard file */
	{
	  rubrica_import_open_file(filename);
	  n = rubrica_vista_get_indirizzi();		  
	}
      break;
      
    case KCARD:
      /* kab (kab 2.x) has a default database */
      buffer =  g_get_home_dir();
      
      filename = g_strdup_printf("%s%s",buffer,card_kde1);
      rubrica_import_open_file(filename);
      titolo = _("Imported from Kab");
      n = rubrica_vista_get_indirizzi();
      break;

    case RUBRICA:
      if (strcmp(filename, "") == 0)
	{
	  GtkWidget *app;
	  
	  app = rubrica_get_app();  
	  gnome_app_error(GNOME_APP(app), _("Error.\nYou must select a file before."));
	  g_error(_("Error.\nYou must select a file before.\n"));
	  return;
	}
      else
      /* apri il file e leggilo */
	{
	  g_print("\nimport %s", filename);
	  n = rubrica_import_open_old_rubrica_file(filename);      
	}
      break;

    case CSV:
    default:
      if (strcmp(filename,"") == 0)
	{
	  GtkWidget *app;
	  
	  app = rubrica_get_app();  
	  gnome_app_error(GNOME_APP(app), _("Error.\nYou must select a file before."));
	  g_error(_("Error.\nYou must select a file before.\n"));
	  return;
	}
      else 
	{
	  g_print("file: %s", filename);
	  rubrica_import_open_file(filename);      
	  titolo = _("Imported from Netscape csv");	  
	  n = rubrica_vista_get_indirizzi();
	}
      break;      
    }

  rubrica_addressbook_set_path_and_name(g_dirname(filename), 
					g_basename(filename), page);
  rubrica_addressbook_set_info(rubrica_get_rubrica(), FALSE, FALSE, FALSE, page);
  
  /* inizializzo nomefile e path (quando si salva 
     il file importato bisogna dargli un nome) */
  rubrica_file_init_names();

  /* visualizzo il numero di indirizzi importati */
  rubrica_vista_set_indirizzi(n);

  /* la rubrica  modificata */
  rubrica_modificata(TRUE);

  rubrica_vista_set_str_label_file(titolo);
  rubrica_set_empty(FALSE);  
}


void rubrica_import_open_file(gchar *file)
{
  FILE *fp;

  if ((fp=fopen(file, "r")) == NULL)
    {
      /* messaggio di errore */ 
      g_print("\nerrore, impossibile leggere il file");
      rubrica_errore_import_no_file();

      return;
    }

  while (!feof(fp))
    {
      gchar buffer[124];

      while (fgets(buffer, 124, fp))
	{	  
	  rubrica_import_gcard_decode_buffer(buffer);
	}
    }
}

void rubrica_export_data(void)
{
}


gint rubrica_import_open_old_rubrica_file(gchar *fname)
{
  rubrica_old_file_apri_file(fname);

  if (rubrica_is_empty())
    return 0;
  
  return (gint) rubrica_vista_get_indirizzi();    
}



