/*
*  Rubrica
*  file: prefer.c
*
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "prefer.h"
#include "dictionary.h"
#include "error.h"
#include "rubrica.h"

RubricaPreferences *pref = NULL;
RubricaPreferences *temp = NULL;
PreferencesBool prefbool;

gboolean load_pref = FALSE;
gboolean default_file_checked = FALSE;
gboolean language_change = FALSE;
gboolean language_code_change = FALSE;

gchar* ab_buffer = NULL;
gchar* lang_buffer = NULL;
gchar* lang_code_buffer = NULL;

void rubrica_preferences_init(gchar *dir, gchar *conf)
{  
  pref = (RubricaPreferences*) g_malloc(sizeof(RubricaPreferences));

  pref->show_path = FALSE;      /* FALSE - show filename, 
				   TRUE  - show path/filename */
  pref->load_ab = TRUE;         /* TRUE  - load default addreesbook when start */
  pref->alpha = TRUE;           /* TRUE  - ordering is alphabetic */
  pref->date = FALSE;           /* TRUE  - oredering is by date (to do) */
  pref->increasing = TRUE;      /* TRUE  - show from minor to major */
  pref->confirm = TRUE;         /* TRUE  - confirm exit from rubrica */
  pref->backup = TRUE;          /* TRUE  - make backup */
  pref->zip = FALSE;            /* TRUE  - gzip addressbook */

  pref->user_dict = FALSE;      /* TRUE  - use user defined dictionary */
  pref->use_english = FALSE;    /* TRUE  - english is default language
				   FALSE - italian is default language */
  pref->dict_loaded = FALSE;    /* TRUE  - user dictionary has been loaded */

  pref->langcode = ITALIAN;          
  pref->language = g_strdup("Italian");
  pref->short_language = g_strdup("IT");

  pref->conf_filename = g_strdup(conf);

  pref->path = g_strdup(g_get_home_dir());  
  pref->ab_name = g_strdup("rubrica.rub");

  pref->dictionary = NULL;

  rubrica_preferences_save(conf);
}

/* temporary preferences, those are filled by user from preferences dialog
   and are copied in pref when apply or ok preferences button is clicked
*/
void rubrica_preferences_temp_init(gchar *conf)
{
  temp->show_path   = FALSE;           	
  temp->load_ab     = FALSE;     
  temp->alpha       = FALSE;              
  temp->date        = FALSE;              
  temp->increasing  = FALSE;            
  temp->confirm     = FALSE;         
  temp->backup      = FALSE;     
  temp->zip         = FALSE;                 
  temp->user_dict   = FALSE;           
  temp->use_english = FALSE;     				
  temp->dict_loaded = FALSE;     

  temp->langcode       = UNKNOWN;          
  temp->language       = NULL;
  temp->short_language = NULL;

  temp->conf_filename = g_strdup(conf);
  temp->path          = NULL; 
  temp->ab_name       = NULL;
  temp->dictionary    = NULL;  

  preferences_init_bool();

}

void preferences_init_bool()
{ 
  prefbool.show       = prefbool.load   = prefbool.alpha    = prefbool.date =  
    prefbool.increase = prefbool.exit   = prefbool.backup   = prefbool.zip  = 
    prefbool.userdict = prefbool.eng    = prefbool.loaddict = prefbool.langcode = 
    prefbool.lang     = prefbool.shlang = prefbool.config   = prefbool.path = 
    prefbool.name     = prefbool.dict   = FALSE; 
}


gint rubrica_preferences_load()
{
  gchar *home_dir;
  gchar *fileconf;
  gchar *dirconf;
  gchar *dirdict;
  
  home_dir = g_get_home_dir();  
  dirconf  = g_strdup_printf("%s/.rubrica", home_dir);
  fileconf = g_strdup_printf("%s/.rubrica/rubrica.conf", home_dir);
  dirdict  = g_strdup_printf("%s/.rubrica/dictionary", home_dir);
 
  temp = (RubricaPreferences*) g_malloc(sizeof(RubricaPreferences));
  rubrica_preferences_temp_init(fileconf);
  
  if (!g_file_exists(fileconf))
    /* first time that rubrica is run, or config file was deleted/moved */
    {
      g_print("\nrubrica initialization...");      

      /* make configuration directory */
      if (!g_file_exists(dirconf))
	mkdir(dirconf, S_IRUSR | S_IWUSR | S_IXUSR);

      /* make dictionary directory */
      if (!g_file_exists(dirdict))
	mkdir(dirdict, S_IRUSR | S_IWUSR | S_IXUSR);      
      
      rubrica_preferences_init(dirconf, fileconf);
      return 0;
    }
  else    /* load preferences from configuration file */
    {
      gboolean err;

      /* need memory */
      pref = (RubricaPreferences*) g_malloc(sizeof(RubricaPreferences));
      
      if (pref == NULL)
	{
	  g_error("Out of memory");
	  exit;
	}

      /*	load preferences
      */
      err = rubrica_load_pref(fileconf);  

      if (err)      /* can't read the configuration file */
	{
	  g_warning("Can't read configuration file. Rubrica will use default option");
	  g_free(pref);

	  rubrica_preferences_init(dirconf, fileconf);
	}

      /*	if user want use his dictionary
       */
      if (pref->user_dict)
	{	  
	  gchar *sh_lang;

	  err = rubrica_load_user_dictionary(pref->language, &sh_lang);
	  
	  if (!err)    /* user dictionary has been readed */
	    {
	      pref->dict_loaded = TRUE; 
	      g_print(_("User dictionary: %s\nLoaded..."), pref->dictionary);
	    }
	  /* error messages have been showed from rubrica_load_dictionary */
	}
      
      /*	 if user want to use englih
      */
      if (pref->use_english)
	{
	  dict_load_en();
	  g_print("English dictionary loaded...");
	}
      else 
	{
	  dict_load_it();
	  g_print("Italian dictionary loaded...");
	}
    }
  load_pref = TRUE;

  g_free(dirconf);
  g_free(fileconf);
  g_free(dirdict);
  
  return 1;
}


gboolean rubrica_load_pref(gchar *file)
{
  xmlDocPtr doc;
  
  doc = xmlParseFile(file);
  
  if (!doc || !doc->root || 
      !doc->root->name || 
      (g_strcasecmp(doc->root->name, "Rubrica_Configuration_File") != 0))
    return TRUE;    
  else
    { 
      /*  here I read the configuration file 
      */ 
      gchar *buffer;
      xmlNodePtr nodo = doc->root->childs; 
      
      if (g_strcasecmp(nodo->name, "Config_data") != 0)
	  return TRUE;
      
      nodo = nodo->childs;

      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->show_path = TRUE;
      else 
	pref->show_path = FALSE; 
      nodo = nodo->next;
   
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->load_ab = TRUE;
      else 
	pref->load_ab = FALSE; 
      nodo = nodo->next;
        
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->alpha = TRUE;
      else 
	pref->alpha = FALSE; 
      nodo = nodo->next;
                
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->date = TRUE;
      else 
	pref->date = FALSE; 
      nodo = nodo->next;
              
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->increasing = TRUE;
      else 
	pref->increasing = FALSE; 
      nodo = nodo->next;
               
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->confirm = TRUE;
      else 
	pref->confirm = FALSE; 
      nodo = nodo->next;
              
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->backup = TRUE;
      else 
	pref->backup = FALSE; 
      nodo = nodo->next;
              
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->zip = TRUE;
      else 
	pref->zip = FALSE; 
      nodo = nodo->next;
              
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->user_dict = TRUE;
      else 
	pref->user_dict = FALSE; 
      nodo = nodo->next;
           
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->use_english = TRUE;
      else 
	pref->use_english = FALSE; 
      nodo = nodo->next;
                           
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->dict_loaded = TRUE;
      else 
	pref->dict_loaded = FALSE; 
      nodo = nodo->next;
              
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "ITALIAN") == 0)
	pref->langcode = ITALIAN;
      else 
	if (g_strcasecmp(buffer, "ENGLISH") == 0)
	  pref->langcode = ENGLISH; 
	else
	  pref->langcode = OTHER;
      nodo = nodo->next;
              
      pref->language = g_strdup(xmlNodeGetContent(nodo));
      nodo = nodo->next;
              
      pref->short_language = g_strdup(xmlNodeGetContent(nodo));
      nodo = nodo->next; 
        
      pref->conf_filename = g_strdup(xmlNodeGetContent(nodo));
      nodo = nodo->next; 
         
      pref->path = g_strdup(xmlNodeGetContent(nodo));
      nodo = nodo->next;       
             
      pref->ab_name = g_strdup(xmlNodeGetContent(nodo));
      nodo = nodo->next;      
              
      pref->dictionary = g_strdup(xmlNodeGetContent(nodo));
    }
  xmlFreeDoc(doc);  
    
  return FALSE;
}

void rubrica_save_preferences()
{
  gchar *filename;

  filename = g_strdup_printf("%s/.rubrica/rubrica.conf", g_get_home_dir());
  
  if (g_file_exists(filename))
    remove(filename);

  rubrica_preferences_save(filename);
  
  g_free(filename);
}

void rubrica_preferences_save(gchar *filename)
{
  xmlDocPtr doc;
  xmlNodePtr newxml;
  xmlNodePtr nodo;

  doc = xmlNewDoc("1.0");

  doc->root = xmlNewDocNode(doc, NULL, "Rubrica_Configuration_File", NULL);

  newxml = xmlNewChild(doc->root, NULL, "Config_data", NULL);
        
  if (pref->show_path)
    nodo = xmlNewTextChild(newxml, NULL, "ShowFileName", "TRUE");
  else
    nodo = xmlNewTextChild(newxml, NULL, "ShowFileName", "FALSE");
 
  if(pref->load_ab)
    nodo = xmlNewTextChild(newxml, NULL, "UseDefaultAddressBook", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "UseDefaultAddressBook", "FALSE");  
    
  if (pref->alpha)
    nodo = xmlNewTextChild(newxml, NULL, "AlphabeticOrdering", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "AlphabeticOrdering", "FALSE");  
   
  if (pref->date)
    nodo = xmlNewTextChild(newxml, NULL, "DateOrdering", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "DateOrdering", "FALSE");  
   
  if (pref->increasing)
    nodo = xmlNewTextChild(newxml, NULL, "ShowIncreasing", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "ShowIncreasing", "FALSE");  
   
  if (pref->confirm)
    nodo = xmlNewTextChild(newxml, NULL, "ConfirmExit", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "ConfirmExit", "FALSE");  
   
  if (pref->backup)
    nodo = xmlNewTextChild(newxml, NULL, "MakeBackup", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "MakeBackup", "FALSE");  
   
  if (pref->zip)
    nodo = xmlNewTextChild(newxml, NULL, "ZipAddressBook", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "ZipAddressBook", "FALSE");  
  
  if (pref->user_dict)
    nodo = xmlNewTextChild(newxml, NULL, "UserDictioanry", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "UserDictionary", "FALSE");  
 
  if (pref->use_english)
    nodo = xmlNewTextChild(newxml, NULL, "UseEnglish", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "UseEnglish", "FALSE");  
      
  if (pref->dict_loaded)
    nodo = xmlNewTextChild(newxml, NULL, "LoadTranslation", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "LoadTranslation", "FALSE");  
   
  switch(pref->langcode)
    {
    case  ITALIAN:
      nodo = xmlNewTextChild(newxml, NULL, "LanguageCode", "ITALIAN");  
      break;
    case ENGLISH:
      nodo = xmlNewTextChild(newxml, NULL, "LanguageCode", "ENGLISH");  
      break;
    case OTHER:
      nodo = xmlNewTextChild(newxml, NULL, "LanguageCode", "OTHER");  
      break;
    default:
      break;
    }
   
  nodo = xmlNewTextChild(newxml, NULL, "Language", pref->language);  
    
  nodo = xmlNewTextChild(newxml, NULL, "ShortLanguage", pref->short_language); 
    
  nodo = xmlNewTextChild(newxml, NULL, "ConfFileName", pref->conf_filename); 
    
  nodo = xmlNewTextChild(newxml, NULL, "Path", pref->path); 
      
  nodo = xmlNewTextChild(newxml, NULL, "AddressBookName", 
			 pref->ab_name);  
    
  nodo = xmlNewTextChild(newxml, NULL, "UserDictionaryName", 
			 pref->dictionary);  
 
  if (xmlSaveFile(filename, doc) == -1) 
    {
      xmlFreeDoc(doc);
      
      g_warning("I can't save configuration file");     
    }
  
  xmlFreeDoc(doc);    
}

/*    functions that manages pref structure
 */
RubricaPreferences *rubrica_get_preferences()
{
  return (RubricaPreferences *) pref;
}

/* ****************************************************** *
 *                                                        *
 *                    SET FUNCTIONS                       *
 *                                                        *
 * ****************************************************** */

void preferences_set_show_path(gboolean show_path)
{
  temp->show_path = show_path;
  prefbool.show=TRUE;
}

void preferences_set_load_ab(gboolean default_ab)
{
  temp->load_ab = default_ab;
  prefbool.load=TRUE;
}

void preferences_set_alpha_order(gboolean ord)
{
  if (ord)
    {
      temp->alpha = TRUE;
      temp->date = FALSE;
    }
  else
    {
      temp->alpha = FALSE;
      temp->date = TRUE;      
    }
  prefbool.alpha = TRUE;
}

void preferences_set_date_order(gboolean ord)
{
  if (ord)
    {
      temp->alpha = FALSE;
      temp->date = TRUE;
    }
  else
    {
      temp->alpha = TRUE;
      temp->date = FALSE;      
    }  
  prefbool.date = TRUE;
}

void preferences_set_increasing(gboolean inc)
{
  temp->increasing = (gboolean) inc;  /* true  show cards from minor to major 
					 false show cards from major to minor */
  prefbool.increase = TRUE;                                       

}

void preferences_set_exit_confirm(gboolean confirm)
{
  temp->confirm = confirm;
  prefbool.exit = TRUE;                                       
}

void preferences_set_do_backup(gboolean bk)
{
  temp->backup = bk;
  prefbool.backup = TRUE;                                       
}

void preferences_set_zip_addressbook(gboolean zip)
{
  temp->zip = zip;
  prefbool.zip = TRUE;                                       
}

void preferences_set_use_dictionary(gboolean use)
{
  temp->user_dict = use;
  prefbool.userdict = TRUE;                                       
}

void preferences_set_use_english(gboolean flag)
{
  temp->use_english = flag;
  prefbool.eng = TRUE;                                       
}

void preferences_set_dictionary_loaded(gboolean load)
{
  temp->dict_loaded = load;
  prefbool.loaddict = TRUE;                                       
}

void preferences_set_lang_code(LANG code)
{
  temp->langcode = code;
  prefbool.langcode = TRUE;                                       
}

void preferences_set_language(gchar* lan)
{
  if (temp->language)
    g_free(temp->language);

  if (lan)
    temp->language = g_strdup(lan);
  else
    temp->language = NULL;
  prefbool.lang = TRUE; 
}

void preferences_set_short_language(gchar* sl)
{
  if (temp->short_language)
    g_free(temp->short_language);
      
  if (sl)
    temp->short_language = g_strdup(sl);
  else 
    temp->short_language = NULL;
  prefbool.shlang = TRUE;                                       
}


void preferences_set_path(gchar *path)
{
  if (temp->path)
    g_free(temp->path);

  if (path)
    temp->path = g_strdup(path);
  else 
    temp->path = NULL; 
  prefbool.path = TRUE;                                       
}

void preferences_set_addressbook_name(gchar *name)
{
  if (temp->ab_name)
    g_free(temp->ab_name);

  if (name)
    temp->ab_name = g_strdup(name);  
  else 
    temp->ab_name = NULL; 
  prefbool.name = TRUE;                                       
}

void preferences_set_dictionary_name(gchar *name)
{
  if (temp->dictionary)
    g_free(temp->dictionary);

  if (name)
    temp->dictionary = g_strdup(name);    
  else 
    temp->dictionary = NULL;
  prefbool.dict = TRUE;                                       
}


/* ****************************************************** *
 *                                                        *
 *                    GET FUNCTIONS                       *
 *                                                        *
 * ****************************************************** */

gboolean preferences_get_show_path(void)
{
  return (gboolean) pref->show_path;
}

gboolean preferences_get_load_ab(void)
{
  return (gboolean) pref->load_ab;
}

gboolean preferences_get_alpha_order(void)
{
  return (gboolean) pref->alpha;
}

gboolean preferences_get_date_order(void)
{
  return (gboolean) pref->date;
}

gboolean preferences_get_increasing(void)
{
  return (gboolean) pref->increasing;
}

gboolean preferences_get_exit_confirm(void)
{
  return (gboolean) pref->confirm;
}

gboolean preferences_get_do_backup(void)
{
  return (gboolean) pref->backup;
}

gboolean preferences_get_zip_addressbook(void)
{
  return (gboolean) pref->zip;
}

gboolean preferences_get_use_dictionary(void)
{
  return (gboolean) pref->user_dict;
}

gboolean preferences_get_use_english(void)
{
  return (gboolean) pref->use_english;
}

gboolean  preferences_get_dictionary_loaded()
{
  return (gboolean) pref->dict_loaded;
}

LANG preferences_get_lang_code(void)
{
  return (LANG) pref->langcode;
}

gchar* preferences_get_language(void)
{
  return (gchar*) pref->language;
}

gchar *preferences_get_short_language(void)
{
  return (gchar*) pref->short_language;
}

gchar *preferences_get_path(void)
{
  return (gchar*) pref->path;
}

gchar *preferences_get_addressbook_name(void)
{
  return (gchar*) pref->ab_name;
}

gchar *preferences_get_dictionary_name(void)
{
  return (gchar*) pref->dictionary;
}


/*
         ***************************************** 
*/
void preferences_set_loaded(gboolean load)
{
  load_pref = load;
}

gboolean preferences_loaded(void)
{
  return (gboolean) load_pref;
}

void preferences_set_def_file_check(gboolean state)
{
  default_file_checked = state;
}

gboolean preferences_get_def_file_check(void)
{
  return (gboolean) default_file_checked;
}


void preferences_lang_changed(gboolean state)
{
  language_change = state;
}

void preferences_lang_code_changed(gboolean state)
{
  language_code_change = state;
}

gboolean preferences_lang_is_changed()
{
  return language_change;
}

gboolean preferences_lang_code_is_changed()
{
  return language_code_change;
}

void preferences_buffer_init()
{
  ab_buffer = lang_buffer = lang_code_buffer = NULL;
}

void preferences_store_default_ab_buffer(gchar* buf)
{
  if (ab_buffer)
    g_free(ab_buffer);

 if (buf) 
   ab_buffer = g_strdup(buf);
 else
   ab_buffer = NULL; 
}

void preferences_store_language_buffer(gchar* buf)
{
  if (lang_buffer)
    g_free(lang_buffer);

  if (buf) 
    lang_buffer = g_strdup(buf);
  else
    lang_buffer = NULL;  
}

void preferences_store_language_code_buffer(gchar* buf)
{
  if (lang_code_buffer)
    g_free(lang_code_buffer);

  if (buf)
    lang_code_buffer = g_strdup(buf);
  else
    lang_code_buffer = NULL; 
}


gchar* preferences_get_ab_buffer()
{
  return (gchar *) ab_buffer;
}

gchar* preferences_get_lang_buffer()
{
  return (gchar *) lang_buffer;
}

gchar* preferences_get_lang_code_buffer()
{
  return (gchar *) lang_code_buffer;
}


/*   *************************************  */

void rubrica_set_prefbox(GtkWidget *w1, GtkWidget *w2, gboolean flag)
{
  if (flag)
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w2), FALSE);
    }
  else
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), FALSE); 
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w2), TRUE);
    }
}


void preferences_refresh_files()
{
  gchar *path=NULL, *file=NULL;

  if (ab_buffer)
    {
      path = g_dirname(ab_buffer);
      file = g_basename(ab_buffer);
    }
      
  preferences_set_addressbook_name(file);
  preferences_set_path(path);
}

/*
  extract preferences from struct and
  fill propertybox's field. 
*/
void preferences_dialog_fill(GtkWidget *prop, RubricaPreferences *p)
{
  GtkWidget *wid1, *wid2, *wid3;


  /*    show path, show filename
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_show_filename");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_show_path");
  if (p->show_path)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);


  /*    load default addressbook when start
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "check_def_addbook");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "fileentry_def_addbook"); 
  wid3 = gtk_object_get_data(GTK_OBJECT(prop), "entry_filename"); 

  if (p->load_ab)
    { 
      /* user want load a default addressbook when rubrica start */
      gchar *str;

      /* extract path and addressbook filename and show them in entry widget */
      str = g_strdup_printf("%s/%s",p->path, p->ab_name);

      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), TRUE);
      gtk_entry_set_text(GTK_ENTRY(wid3), str);        
      gtk_widget_set_sensitive(GTK_WIDGET(wid2), TRUE);  

      g_free(str);
    }
  else 
    {
      /* not load addressbook at rubrica start */
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), FALSE);
      gtk_entry_set_text(GTK_ENTRY(wid3), "");             /* clear entry */
      gtk_widget_set_sensitive(GTK_WIDGET(wid2), FALSE);                  
    }


  /*    sort by: date, alphabetic
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_date");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_alpha");
  if (p->alpha)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    if (p->date)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);


  /*    sort: increasing, decreasing
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_dec");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_inc");
  if (p->increasing)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);


  /*    make backup: never, always
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_never");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_always");
  if (p->backup)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);


  /*    confirm exit: never, always
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_confirm_no");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_confirm_yes");
  if (p->confirm)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);


  /*    zip addressbook: yes, no    
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_zip_no");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_zip_yes");
  if (p->zip)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);

  
  /*    use user dictionary: yes, no	
	set default language to
	1. user language
	2. english
	3. italian
  */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_other_lang");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_def_lang");
  if (p->user_dict)
    {
      GtkWidget *wid4, *wid5, *wid6, *wid7;

      /*    set user language and language code
       */
      wid4 = gtk_object_get_data(GTK_OBJECT(prop), "entry_user_lang");
      wid5 = gtk_object_get_data(GTK_OBJECT(prop), "entry_user_lang_code");
      wid6 = gtk_object_get_data(GTK_OBJECT(prop), "radio_def_ita");
      wid7 = gtk_object_get_data(GTK_OBJECT(prop), "radio_def_eng");

      gtk_entry_set_text(GTK_ENTRY(wid4), p->language);
      gtk_entry_set_text(GTK_ENTRY(wid5), p->short_language);

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(wid6), FALSE);
      gtk_widget_set_sensitive(GTK_WIDGET(wid7), FALSE);

      /* widgets that are related to this radio button, will be 
	 activated in the callback called when radio button 
	 receive the toggle signal. 
      */
    }
  else
    {
      GtkWidget *wid4, *wid5, *wid6, *wid7;

      /*    clear language and language code
       */
      wid4 = gtk_object_get_data(GTK_OBJECT(prop), "entry_user_lang");
      wid5 = gtk_object_get_data(GTK_OBJECT(prop), "entry_user_lang_code");

      gtk_entry_set_text(GTK_ENTRY(wid4), "");
      gtk_entry_set_text(GTK_ENTRY(wid5), "");

      /*	set default language to italian or english
       */
      wid6 = gtk_object_get_data(GTK_OBJECT(prop), "radio_def_ita");
      wid7 = gtk_object_get_data(GTK_OBJECT(prop), "radio_def_eng");
      
      gtk_widget_set_sensitive(GTK_WIDGET(wid6), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(wid7), TRUE);
      if (p->use_english)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid7), TRUE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid6), TRUE);

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
    }
}


void preferences_apply_changes(void)
{
  if (prefbool.show)
    pref->show_path = temp->show_path; 
  if (prefbool.load)
    pref->load_ab = temp->load_ab;
  if (prefbool.alpha)  
    pref->alpha = temp->alpha;          
  if (prefbool.date)
    pref->date = temp->date;        
  if (prefbool.increase)
    pref->increasing = temp->increasing;     
  if (prefbool.exit)
    pref->confirm = temp->confirm; 
  if (prefbool.backup)
    pref->backup = temp->backup; 
  if (prefbool.zip)
    pref->zip =  temp->zip;          
  if (prefbool.userdict) 
    pref->user_dict = temp->user_dict;   
  if (prefbool.eng)
    pref->use_english = temp->use_english;			
  if (prefbool.loaddict)
    pref->dict_loaded = temp->dict_loaded;
  
  if (prefbool.langcode)
    pref->langcode = temp->langcode; 
 
  if (prefbool.lang)
    {
      if (pref->language)
	g_free(pref->language);
      
      if (temp->language)
	pref->language = g_strdup(temp->language); 
      else
	pref->language = NULL;	
    }
  if (prefbool.shlang) 
    {
      if (pref->short_language)    
	g_free(pref->short_language);
      
      if (temp->short_language)
	pref->short_language = g_strdup(temp->short_language);
      else
	pref->short_language = NULL;	
    }
/*
  if (prefbool.config)
    pref->conf_filename = temp->conf_filename;  
*/
  if (prefbool.path)
    {
      if (pref->path)
	g_free(pref->path);

      if (temp->path)
	pref->path = g_strdup(temp->path);
      else
	pref->path = NULL;  
    }

  if (prefbool.name)
    {
      if (pref->ab_name)
	g_free(pref->ab_name );

      if (temp->ab_name )
	pref->ab_name = g_strdup(temp->ab_name );
      else
	pref->ab_name = NULL;      
    }

  if (prefbool.dict)
    {
      if (pref->dictionary )
	g_free(pref->dictionary);

      if (temp->dictionary)
	pref->dictionary = g_strdup(temp->dictionary);
      else
	pref->dictionary = NULL;  
    }
  
  preferences_init_bool();
}
