/*
*  Rubrica
*  file: prefer.h
*
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_PREF_H
#define _RUBRICA_PREF_H

#include "dictionary.h"

typedef struct _RubricaPreferences {
  gboolean show_path;     /* false,  show only filename
			     true  , show path and filename */
  gboolean load_ab;       /* true, rubrica must load the setted addressbook at start 
			     false, rubrica begin with clean screen */
  gboolean alpha;         /* show cards in alphabetic order */
  gboolean date;          /* show cards by creation date */
  gboolean increasing;    /* true, show cards fron minor to major */  
  gboolean confirm;       /* true, confirn exit */ 
  gboolean backup;        /* true, make backup */
  gboolean zip;           /* true, zip address book */

  gboolean user_dict;     /* true, load user dictionary */
  gboolean use_english;   /* default language are "It" or "En"
			     false - use italian
			     true  - use english */
  gboolean dict_loaded;   /* user's dictionary has been loaded */

  LANG langcode;          /* code language */
  gchar *language;        /* language used */
  gchar *short_language;  /* string code language */

  gchar *conf_filename;   /* config filename */

  gchar *path;            /* path of addressbook that user want load at start */
  gchar *ab_name;         /* name of addressbook that user want load at start */

  gchar *dictionary;      /* user's dictionary name */
} RubricaPreferences;

typedef struct _PreferencesBool {
  gboolean show;
  gboolean load;
  gboolean alpha;
  gboolean date;
  gboolean increase;
  gboolean exit;
  gboolean backup;
  gboolean zip;
  gboolean userdict;
  gboolean eng;
  gboolean loaddict;
  gboolean langcode;
  gboolean lang;
  gboolean shlang;
  gboolean config;
  gboolean path;
  gboolean name;
  gboolean dict;
} PreferencesBool;

void rubrica_preferences_init(gchar *dir, gchar *conf);
void rubrica_preferences_temp_init(gchar *conf);
void preferences_init_bool(void);

gint rubrica_preferences_load();
gboolean rubrica_load_pref(gchar *file);
void rubrica_preferences_save(gchar *filename);
void rubrica_save_preferences(void);

RubricaPreferences *rubrica_get_preferences(void);

void preferences_apply_changes(void);
/* ****************************************************** *
 *                                                        *
 *                    SET FUNCTIONS                       *
 *                                                        *
 * ****************************************************** */

/*      functions that manages pref structure
	set boolean values
*/
void preferences_set_show_path(gboolean show_file);
void preferences_set_load_ab(gboolean default_ab);
void preferences_set_alpha_order(gboolean ord);
void preferences_set_date_order(gboolean ord);
void preferences_set_increasing(gboolean inc);
void preferences_set_exit_confirm(gboolean confirm);
void preferences_set_do_backup(gboolean bk);
void preferences_set_zip_addressbook(gboolean zip);
void preferences_set_use_dictionary(gboolean use);
void preferences_set_use_english(gboolean use_eng);
void preferences_set_dictionary_loaded(gboolean load);

/*      set language values      */
void preferences_set_lang_code(LANG code);
void preferences_set_language(gchar*);
void preferences_set_short_language(gchar*);

/*      set path/filenames values     */
void preferences_set_path(gchar *path);
void preferences_set_addressbook_name(gchar *name);
void preferences_set_dictionary_name(gchar *name);

/* ****************************************************** *
 *                                                        *
 *                    GET FUNCTIONS                       *
 *                                                        *
 * ****************************************************** */

/*      get boolean values     */
gboolean preferences_get_show_path(void);
gboolean preferences_get_load_ab(void);
gboolean preferences_get_alpha_order(void);
gboolean preferences_get_date_order(void);
gboolean preferences_get_increasing(void);
gboolean preferences_get_exit_confirm(void);
gboolean preferences_get_do_backup(void);
gboolean preferences_get_zip_addressbook(void);
gboolean preferences_get_use_dictionary(void);
gboolean preferences_get_use_english(void);
gboolean preferences_get_dictionary_loaded();

LANG preferences_get_lang_code(void);
gchar* preferences_get_language(void);
gchar* preferences_get_short_language(void);

gchar* preferences_get_path(void);
gchar* preferences_get_addressbook_name(void);
gchar* preferences_get_dictionary_name(void);
/* **************************************************** */


/*  get - set globals variables*/
void preferences_set_loaded(gboolean load);
gboolean preferences_loaded(void);

void preferences_set_def_file_check(gboolean state);
gboolean preferences_get_def_file_check(void);

#define dictionary_is_loaded() preferences_get_dictionary_loaded()

void preferences_lang_changed(gboolean state);
void preferences_lang_code_changed(gboolean state);
gboolean preferences_lang_is_changed();
gboolean preferences_lang_code_is_changed();

void preferences_buffer_init();
void preferences_store_default_ab_buffer(gchar* buf);
void preferences_store_language_buffer(gchar* buf);
void preferences_store_language_code_buffer(gchar* buf);
gchar* preferences_get_ab_buffer();
gchar* preferences_get_lang_buffer();
gchar* preferences_get_lang_code_buffer();
/*  *************************************  */

void rubrica_set_prefbox(GtkWidget *w1, GtkWidget *w2, gboolean flag);

void preferences_refresh_files();
void preferences_dialog_fill(GtkWidget *prop, RubricaPreferences *p);



#endif






