/*
*  Rubrica
*  file: rubrica.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "rubrica.h"
#include "view.h"
#include "modulo.h"
#include "prefer.h"
#include "main.h"
#include "file.h"
#include "addressbook.h"


GList *rubrica = NULL;                   /* rubrica is one of book items */
RubricaItem *rtemp = NULL;
gboolean modificata = FALSE;
gboolean rubrica_empty = TRUE;
gboolean new_ab = TRUE;
gint current_page = 0;


GList *rubrica_get_rubrica(void)
{
  return (GList*) rubrica;
}

void rubrica_set_rubrica(gpointer ptr)
{
  rubrica = ptr;
}


void
rubrica_inserisci_scheda(RubricaItem *r)
{
  gchar *buffer[5];
  VistaDati vista;
  InfoRubrica info;
  
  g_return_if_fail(r != NULL);

  buffer[0] = "";                    // cella per l'eventuale pixmap
  buffer[1] = g_strdup(r->scheda);
  buffer[2] = g_strdup(r->nome);
  buffer[3] = g_strdup(r->cognome);
  buffer[4] = NULL;

  rubrica_get_vista(&vista);

  gtk_progress_set_value(GTK_PROGRESS(vista.progressbar), 
			 (float) vista.totale_indirizzi);
  rubrica_vista_get_info(&info, VISTA_SCHEDE);
  if(info.maxrow < vista.totale_indirizzi)
    info.maxrow = vista.totale_indirizzi;
  rubrica_vista_set_info(info, VISTA_SCHEDE);
  gtk_clist_append(GTK_CLIST(vista.schede), buffer);
}

void 
rubrica_elimina_scheda(gint n)
{
  VistaDati vista;
  InfoRubrica info;

  rubrica_get_vista(&vista);
  
  /* il numero degli elementi  stato aggiornato nella 
     chiamata a rubrica_elimina_elemento
  */
  gtk_clist_remove(GTK_CLIST(vista.schede), n);  
  gtk_progress_set_value(GTK_PROGRESS(vista.progressbar), 
			 (float) vista.totale_indirizzi);  
  rubrica_vista_get_info(&info, VISTA_SCHEDE);
  if(info.maxrow > vista.totale_indirizzi)
    info.maxrow = vista.totale_indirizzi;
  rubrica_vista_set_info(info, VISTA_SCHEDE);
}

void
rubrica_modifica_scheda(RubricaItem *r)
{
  VistaDati vista;
  gchar *buffer[5];
  gint n;

  g_return_if_fail(r != NULL);

  rubrica_get_vista(&vista);

  buffer[0] = "";                    // cella per l'eventuale pixmap
  buffer[1] = g_strdup(r->scheda);
  buffer[2] = g_strdup(r->nome);
  buffer[3] = g_strdup(r->cognome);
  buffer[4] = NULL;
  
  n = vista.schedeinfo.riga;

  gtk_clist_remove(GTK_CLIST(vista.schede), n);
  gtk_clist_insert(GTK_CLIST(vista.schede), n, buffer);
}

/* gestione elementi
*/
inline gboolean rubrica_is_modificata()
{
  return (gboolean) modificata;
}


void rubrica_modificata(gboolean stato)
{
  GtkWidget *win;

  win = rubrica_get_app();

  if (stato) /* modificata */
    {
      gtk_window_set_title(GTK_WINDOW(win), _("Rubrica (changed)"));
      rubrica_set_titolo(_("Rubrica (changed)"));        
    }
  else      /* non modificata */
    {
      gtk_window_set_title(GTK_WINDOW(win), "Rubrica");
      rubrica_set_titolo("Rubrica");         
    }
  
  modificata = stato;
}

/*   insert an item in addressbook
*/
void rubrica_inserisci_elemento(RubricaItem *r)
{
  rubrica = g_list_append(rubrica, r);
  
  if (rubrica_is_new())
    {
      gint page;

      page = rubrica_get_current_page();

      rubrica_addressbook_set_rubrica(rubrica, page);
    }    
  rubrica_set_new(FALSE);
}

/*  return the nth item in addressbook 
*/
RubricaItem *rubrica_prendi_elemento(gint n)
{
  return (RubricaItem *) g_list_nth_data(rubrica, n);
}


/*    modify a rubrica's item
*/
gint rubrica_modifica_elemento(RubricaItem *r)
{
  gint n;
  GList *e = NULL;  
  VistaDati vista;
  gpointer new, old;

  g_return_val_if_fail(r != NULL, 0);

  rubrica_get_vista(&vista);

  new = r;
  n = vista.schedeinfo.riga;

  e = g_list_nth(rubrica, n);
  old = g_list_nth_data(rubrica,n);
  
  e->data = new;
  g_free(old);
  
  return 1;
}


/*      free memory used by rubrica's item
*/
void rubrica_free_data(gpointer data, gpointer user_data)
{  
  g_return_if_fail(data != NULL);
  g_return_if_fail(user_data != NULL);
  
  if (user_data)
    g_free(user_data);
}

/*      get the item that will be deleted
 */
gint rubrica_elimina_elemento(GList *rubrica, gint n)
{
  /*
    NOTA:
    La memoria occupata dalle liste si libera
    con g_list_free_1 e non con g_free() 
    pena SEGMENTATION FAULT
  */
  GList *nodo;
  GList *wl, *ml, *tl;
  RubricaItem *r;

  g_return_val_if_fail(rubrica != NULL, 0);

  /* prendo l'elemento alla posizione n */
  nodo = g_list_nth(rubrica, n);
  
  /* rimuovo l'elemento nodo dalla lista
     la memoria occupata da nodo ancora non  liberata. 
     Ora nodo  la testa di una nuova lista, posso aggiungere 
     altri elementi e costruire una nuova lista,
     o cancellarlo elimindolo dalla lista
  */
  rubrica = g_list_remove_link(rubrica, nodo);

  /* estraggo i dati contenuti nel nodo e libero 
     la memoria che occupano 
  */
  r = (RubricaItem*) g_list_nth_data(nodo, 0);

  /*
    r  un ptr a RubricaItem, 
    che contiene diversi *gchar e 3 liste.   
  */
  wl = r->web_list;
  ml = r->mail_list;
  tl = r->tel_list;

  /* ridondanza di controlli??
  	 preferisco effettuare pi controlli ed evitare
	 segmentation fault. Se qualcuno vuole reimplemantare
	 in modo migliore questa parte di codice,  il
	 benvenuto!! 

	 If someone thinks that there are much controls, 
	 well i think so. But i prefer this to a segmentation fault.
	 Enyone can modify this code, to make it better. Thanks :-)	 

   */
  if (wl) {
    g_list_foreach(wl, rubrica_free_data, wl->data);
    g_list_free_1(r->web_list); 
  }
  
  if (ml) {
    g_list_foreach(ml, rubrica_free_data, ml->data);
    g_list_free_1(r->mail_list);
  }
  
  if (tl) {
    g_list_foreach(tl, rubrica_free_data, tl->data);
    g_list_free_1(r->tel_list);
  }
  
  if(r->scheda)
    g_free(r->scheda);
  if(r->nome)
    g_free(r->nome);
  if(r->cognome)
    g_free(r->cognome);
  if(r->professione)
    g_free(r->professione);
  if(r->via)
    g_free(r->via);
  if(r->numero)
    g_free(r->numero);
  if(r->cap)
    g_free(r->cap);
  if(r->citta)
    g_free(r->citta);
  if(r->provincia)
    g_free(r->provincia);
  if(r->stato)
    g_free(r->stato);
  if(r->note)
    g_free(r->note);

  g_free(r);

  /* libero la menoria occupata da nodo */
  g_list_free_1(nodo);

  /* decremento il numero totale di indirizzi */
  rubrica_vista_dec_indirizzi();
  return 1;
}

void rubrica_svuota_memoria(GList *rubrica)
{
  gint n;
  
  n = g_list_length(rubrica);
  
  while (n)
    {
      rubrica_elimina_elemento(rubrica, n-1);
      n--;
    }      
  rubrica = NULL;
}


/* gestione visualizzazione dati schede
   
   TO DO:

   da spostare in un file dedicato alla visualizzazione dei dati 
   contenuti in rubrica
*/
void rubrica_mostra_scheda(RubricaItem *i)
{
  GtkCList *clist;
  GList *alias = NULL;
  gint n = 11;
  gint elementi = 0;
  gboolean modificato;              

  /* dati[..][0]  riservato per probabili future immagini
     dati[..][0] is reserved. Probably future pixmap
   */
  gchar *dati[19][4] = {
    {NULL, _("Card: "),              NULL, NULL},        /*  0 =>  1  */
    {NULL, _("First Name: "),        NULL, NULL},        
    {NULL, _("Last Name: "),         NULL, NULL},
    {NULL, _("Profession: "),        NULL, NULL},
    {NULL, _("Street: "),            NULL, NULL},
    {NULL, _("Number: "),            NULL, NULL},
    {NULL, _("Zip code: "),          NULL, NULL},
    {NULL, _("City: "),              NULL, NULL},
    {NULL, _("Province: "),          NULL, NULL},
    {NULL, _("Country: "),           NULL, NULL},
    {NULL, _("Web Pages: "),         NULL, NULL},        /*  10 => 11 */
    {NULL, _("Email Address: "),     NULL, NULL},
    {NULL, _("Home Telephone: "),    NULL, NULL},
    {NULL, _("Work Telephone: "),    NULL, NULL},
    {NULL, _("Fax Telephone: "),     NULL, NULL},
    {NULL, _("Cellphone: "),         NULL, NULL},
    {NULL, _("Other Telephone: "),   NULL, NULL},
    {NULL, _("Notes: "),             NULL, NULL},
    {NULL, NULL,                     NULL, NULL}         /*  18  => 19 */
  };
  
  g_return_if_fail(i != NULL);

  clist = rubrica_vista_get_dati();
  gtk_clist_clear(GTK_CLIST(clist));

  dati[0][2] = g_strdup(i->scheda);
  dati[1][2] = g_strdup(i->nome);
  dati[2][2] = g_strdup(i->cognome);
  dati[3][2] = g_strdup(i->professione);
  dati[4][2] = g_strdup(i->via);
  dati[5][2] = g_strdup(i->numero);
  dati[6][2] = g_strdup(i->cap);
  dati[7][2] = g_strdup(i->citta);
  dati[8][2] = g_strdup(i->provincia);
  dati[9][2] = g_strdup(i->stato);
  
  rubrica_visualizza_dati(clist, dati);
  
  /*   web   */
  alias = g_list_first(i->web_list);
  elementi = i->nweb;
  modificato = FALSE;
  while (elementi > 0)
    {
      if (alias->data)
	dati[10][2] = g_strdup(alias->data);  

      gtk_clist_insert(GTK_CLIST(clist),n, dati[10]);
      if (!modificato)
	{
	  dati[10][1] = g_strdup("");
	  modificato = TRUE;
	}
      n++;
      elementi--;
      alias = g_list_next(alias);
    }

  /*   mail   */
  alias = g_list_first(i->mail_list);
  modificato = FALSE;
  elementi = i->nmail;
  while (elementi > 0)
    {
      if (alias->data)
	dati[11][2] = g_strdup(alias->data);  

      gtk_clist_insert(GTK_CLIST(clist),n, dati[11]);
      if (!modificato)
	{
	  dati[11][1] = g_strdup("");
	  modificato = TRUE;
	}
      n++;
      elementi--;
      alias = g_list_next(alias);
    }

  /*   telefono   */
  alias = g_list_first(i->tel_list);
  elementi = i->ntel;
  while (elementi > 0)
    {
 
      TelNum *tel = NULL;
      TIPO_TELEFONO tipo = TELEFONO_SCONOSCIUTO;

      if (alias->data)     
	{
	  tel = (TelNum*) alias->data; 
	  tipo = tipo_telefono_int(tel->tipo);
	}
      
      switch (tipo)
        {
        case TELEFONO_CASA:	  
          dati[12][2] = g_strdup(tel->numero_telefono);
          gtk_clist_insert(GTK_CLIST(clist),n, dati[12]);
          n++;
	  elementi--;
	  alias = g_list_next(alias);
          break;
        case TELEFONO_UFFICIO:
          dati[13][2] =  g_strdup(tel->numero_telefono);
          gtk_clist_insert(GTK_CLIST(clist),n, dati[13]);
          n++;
	  elementi--;
	  alias = g_list_next(alias);
          break;
        case TELEFONO_FAX:
          dati[14][2] = g_strdup(tel->numero_telefono);
          gtk_clist_insert(GTK_CLIST(clist),n, dati[14]);
          n++;
	  elementi--;
	  alias = g_list_next(alias);
          break;
        case TELEFONO_CELLULARE:
          dati[15][2] = g_strdup(tel->numero_telefono);
          gtk_clist_insert(GTK_CLIST(clist),n, dati[15]);
          n++;
	  elementi--;
	  alias = g_list_next(alias);
        case TELEFONO_ALTRO:
          dati[16][2] = g_strdup(tel->numero_telefono);
          gtk_clist_insert(GTK_CLIST(clist),n, dati[16]);
          n++;
	  elementi--;
	  alias = g_list_next(alias);
          break;
        default:
          break;
        }
    }  
  if (i->note)
    dati[17][2] = g_strdup(i->note);
  
  gtk_clist_insert(GTK_CLIST(clist), n, dati[17]);  
}


/*   mostra i dati contenuti nella tabella "dati"
     dalla riga 0 alla riga 10
*/
void rubrica_visualizza_dati(GtkCList *clist, gchar *item[18][4])
{
  gint i;

  for (i=0; i<10; i++)
    gtk_clist_insert(GTK_CLIST(clist),i, item[i]);
}


/*  ----------------------------------------------------------------- */

/* 
   routine per la nagivazione delle schede inserite
   visualizzano la:
    - scheda precedente           (click pre)
    - scheda successiva           (click succ)
    - ultima scheda               (click prima)
    - prima scheda                (click ultima)
*/
void rubrica_mostra_prima_scheda()
{
  RubricaItem *i;
  VistaDati vista;
  GtkCList *clist;

  g_return_if_fail(rubrica != NULL);

  rubrica_get_vista(&vista);
  g_return_if_fail(vista.totale_indirizzi > 0);

  i = g_list_nth_data(rubrica,0);
  
  g_return_if_fail(i != NULL);
  
  clist = rubrica_vista_get_schede();
  gtk_clist_unselect_row(GTK_CLIST(clist), 0,0);

  rubrica_mostra_scheda(i);
  rubrica_set_scheda_corrente(0);
}

void rubrica_mostra_ultima_scheda()
{
  guint n;
  RubricaItem *i;
  GtkCList *clist;
  InfoRubrica info;
  VistaDati vista;
  
  g_return_if_fail(rubrica != NULL);

  rubrica_get_vista(&vista);
  g_return_if_fail(vista.totale_indirizzi > 0);

  /* se la scheda contiene n elementi, l'ultimo elemento
     della lista  in posizione n-1.
     es. 
     posizione :  0  1  2  3
     elemento  :  1  2  3  4
  */
  n = g_list_length(rubrica)-1;
  i = g_list_nth_data(rubrica,n);

  g_return_if_fail(i != NULL);

  rubrica_vista_get_info(&info, VISTA_SCHEDE);
  clist = rubrica_vista_get_schede();
  gtk_clist_unselect_row(GTK_CLIST(clist), info.riga, info.colonna);

  rubrica_mostra_scheda(i);
  rubrica_set_scheda_corrente(n);
}


void rubrica_scheda_precedente()
{
  guint n;
  GtkCList *clist;
  RubricaItem *i;
  VistaDati vista;
  InfoRubrica info;

  rubrica_vista_get_info(&info, VISTA_SCHEDE);
  n = rubrica_get_scheda_corrente()-1;
  rubrica_get_vista(&vista);
  
  g_return_if_fail(rubrica != NULL);
  g_return_if_fail(n >= 0);
  g_return_if_fail(vista.totale_indirizzi > 0);

  i = g_list_nth_data(rubrica,n);
  
  g_return_if_fail(i != NULL);

  clist = rubrica_vista_get_schede();
  gtk_clist_unselect_row(GTK_CLIST(clist), info.riga, info.colonna);

  rubrica_mostra_scheda(i);
  rubrica_set_scheda_corrente(n);
}


void rubrica_scheda_successiva()
{
  guint n, tot;
  GtkCList *clist;
  RubricaItem *i;
  VistaDati vista;
  InfoRubrica info;
  
  rubrica_get_vista(&vista);
  rubrica_vista_get_info(&info, VISTA_SCHEDE);
  tot = rubrica_vista_get_indirizzi();
  n = rubrica_get_scheda_corrente()+1;
  
  g_return_if_fail(rubrica != NULL);
  g_return_if_fail(vista.totale_indirizzi > 0);
  g_return_if_fail(n <= tot);

  i = g_list_nth_data(rubrica,n);

  g_return_if_fail(i != NULL);

  clist = rubrica_vista_get_schede();
  gtk_clist_unselect_row(GTK_CLIST(clist), info.riga, info.colonna);

  rubrica_mostra_scheda(i);
  rubrica_set_scheda_corrente(n);
}

/*         da spostare in un file per la visualizzazione        */
/* ************************************************************ */


/*  ------------------------------------------------------------------ */

/* fuzioni da spostare nel file delle preferenze */

/*
inline void rubrica_set_mostra_path(gboolean stato)
{
  preferences_set_show_path(stato);
}

inline gboolean rubrica_get_mostra_path(void)
{
  return (gboolean) preferences_get_show_path();
}
*/

/* ********************************************* */

inline
gboolean rubrica_is_empty(void)
{
  return (gboolean) rubrica_empty;
}

inline 
void rubrica_set_empty(gboolean stato)
{
  rubrica_empty = stato;
}

inline 
void rubrica_set_current_page(gint p)
{
  current_page = p;
}

inline 
gint rubrica_get_current_page(void)
{
  return (gint) current_page;
}

inline 
void rubrica_init_globals(void)
{
  rubrica_set_rubrica(NULL);
  rubrica_modificata(FALSE);
  rubrica_set_empty(TRUE);
  rubrica_file_init_names();
  rubrica_vista_set_indirizzi(0);  
}

inline
gboolean rubrica_is_new()
{
  return (gboolean) new_ab;
}

inline
void rubrica_set_new(gboolean state)
{
  new_ab = (gboolean) state;
}


/*    cut copy paste 
*/
void rubrica_item_copy(GList *rubrica, gint riga)
{
  RubricaItem *r;
  
  g_return_if_fail (rubrica != NULL);
  g_return_if_fail (riga >= 0);

  rtemp = g_malloc(sizeof(RubricaItem));
  r = (RubricaItem*) g_list_nth_data(rubrica, riga);
  
  rtemp->web_list = rtemp->mail_list = rtemp->tel_list = NULL;

  rtemp->editabile = r->editabile;
  rtemp->cancellabile = r->cancellabile;
  
  rtemp->nweb  = r->nweb;
  rtemp->nmail = r->nmail;
  rtemp->ntel  = r->ntel;
  
  rtemp->scheda      = g_strdup(r->scheda);
  rtemp->nome        = g_strdup(r->nome);
  rtemp->cognome     = g_strdup(r->cognome);
  rtemp->professione = g_strdup(r->professione);
  rtemp->via         = g_strdup(r->via);
  rtemp->numero      = g_strdup(r->numero);
  rtemp->cap         = g_strdup(r->cap);
  rtemp->citta       = g_strdup(r->citta);
  rtemp->provincia   = g_strdup(r->provincia);
  rtemp->stato       = g_strdup(r->stato);
  rtemp->note        = g_strdup(r->note);

  while (r->web_list)
    {
      rtemp->web_list = g_list_append(rtemp->web_list, r->web_list->data);
      r->web_list = g_list_next(r->web_list);
    }

  while (r->mail_list)
    {
      rtemp->mail_list = g_list_append(rtemp->mail_list, r->mail_list->data);
      r->mail_list = g_list_next(r->mail_list);
    }  

  while (r->tel_list)
    {
      rtemp->tel_list = g_list_append(rtemp->tel_list, r->tel_list->data);
      r->tel_list = g_list_next(r->tel_list);
    }

  /* aggiungere il codice per inserire in undo/redo la "copiatura dell'elemento */
}

void rubrica_item_cut(GList *rubrica, gint riga)
{
  GList *nodo;

  g_return_if_fail (rubrica != NULL);
  g_return_if_fail (riga >= 0);

  nodo = g_list_nth(rubrica, riga);
  rubrica = g_list_remove_link(rubrica, nodo);
  rtemp = (RubricaItem*) g_list_nth_data(nodo, 0);
  rubrica_vista_dec_indirizzi();
}

void rubrica_item_paste(GList *rubrica, gint pos)
{
  GtkWidget *errdlg;
  InfoRubrica info;
  VistaDati vista;
  gchar *buffer[5];
  gint list_len;
  gchar *err;
  
  g_return_if_fail (pos >= 0);

  if (!rtemp)
    {
      err = g_strdup(_("I can't paste anything if you don't cut/copy before"));
      errdlg = gnome_error_dialog(err);
      gtk_widget_show(errdlg);
    }
  
  buffer[0] = "";                    // cella per l'eventuale pixmap
  buffer[1] = g_strdup(rtemp->scheda);
  buffer[2] = g_strdup(rtemp->nome);
  buffer[3] = g_strdup(rtemp->cognome);
  buffer[4] = NULL;

  list_len = g_list_length(rubrica);

  /* rubrica = ab->rubrica
   */
  rubrica_vista_get_info(&info, VISTA_SCHEDE);
  rubrica_get_vista(&vista);
      
  if(info.maxrow < vista.totale_indirizzi)
    {
      info.maxrow = vista.totale_indirizzi;
      rubrica_vista_set_info(info, VISTA_SCHEDE);
    }

  /* if list_len == 0 then rubrica == NULL */
  if (pos > list_len)
    {
      gtk_clist_append(GTK_CLIST(vista.schede), buffer);
      rubrica_inserisci_elemento(rtemp);  
    }
  else
    {
      gtk_clist_insert(GTK_CLIST(vista.schede), pos, buffer);
      rubrica = g_list_insert(rubrica, rtemp, pos);
    }
 
  /* list_len non mi serve pi, quindi riuso la variabile 
     I don't need list_len, reuse the variable 
  */
  list_len = rubrica_vista_get_indirizzi();
  rubrica_vista_set_indirizzi(list_len+1);
}
