/*
*  Rubrica
*  file: rubrica.h
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _RUBRICA_H
#define _RUBRICA_H

#include <gnome.h>

#include "view.h"

typedef enum  {
  TELEFONO_CASA = 0,
  TELEFONO_UFFICIO,
  TELEFONO_FAX,
  TELEFONO_CELLULARE,
  TELEFONO_ALTRO,
  TELEFONO_SCONOSCIUTO
} TIPO_TELEFONO;

typedef struct  {
  gboolean editabile;
  gboolean cancellabile;

  gchar *scheda;
  gchar *nome;
  gchar *cognome;
  gchar *professione;
  gchar *via;
  gchar *numero;
  gchar *cap;
  gchar *citta;
  gchar *provincia;
  gchar *stato;
  gchar *note;

  GList *web_list;
  GList *mail_list;
  GList *tel_list;

  gint nweb;
  gint nmail;
  gint ntel;

} RubricaItem;

typedef struct _TelNum {
  gchar *numero_telefono;
  gchar *tipo;
} TelNum;



/* *********************************************************************** *
   
   Prototipi

 * *********************************************************************** */


/* ***********************************************
    prelievo e impostazione della lista a rubrica
*/
GList *rubrica_get_rubrica(void);
void rubrica_set_rubrica(gpointer ptr);

void rubrica_init_globals(void);

/* ************************************************** *
  metodi per l'inserimento, cancellazione e modifica 
  di un elemento di rubrica nella lista rubrica       */
void rubrica_inserisci_scheda(RubricaItem *r);
void rubrica_modifica_scheda(RubricaItem *r);
void rubrica_elimina_scheda(gint n);

void rubrica_svuota_memoria(GList *rubrica);
void rubrica_free_data(gpointer data, gpointer user_data);

RubricaItem *rubrica_prendi_elemento(gint n);
void rubrica_inserisci_elemento(RubricaItem *r);
gint rubrica_modifica_elemento(RubricaItem *r);
gboolean rubrica_elimina_elemento(GList *rubrica, gint n);

inline void rubrica_modificata(gboolean stato);
inline gboolean rubrica_is_modificata(void);
inline void rubrica_set_empty(gboolean stato);
inline gboolean rubrica_is_empty(void);
/* *********************************************** */

/* **************************** *
   funzioni per la nagigazione
   tra le schede inserite       */
void rubrica_mostra_prima_scheda();
void rubrica_mostra_ultima_scheda();

void rubrica_scheda_precedente();
void rubrica_scheda_successiva();

/* ****************************************** */

void rubrica_mostra_scheda(RubricaItem *i);
void rubrica_visualizza_dati(GtkCList *clist, gchar *item[18][4]);

/* ***************************************** 
   funzioni per l' inserimento e 
   prelienvo della path corrente
*/ 
inline gboolean rubrica_get_mostra_path(void);
inline void rubrica_set_mostra_path(gboolean stato);

/* ****************************************** */

/*
  funzioni per:
  taglia, copia, incolla (cut, copy, paste)
*/
void rubrica_item_cut(GList *rubrica, gint riga);
void rubrica_item_copy(GList *rubrica, gint riga);
void rubrica_item_paste(GList *rubrica, gint pos);

/* ****************************************** */

inline void rubrica_set_current_page(gint p);
inline gint rubrica_get_current_page(void);

inline gboolean rubrica_is_new();
inline void rubrica_set_new(gboolean state);

#endif










